package converter;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.StringTokenizer;

import org.eclipse.emf.common.util.EList;

import ru.novosoft.uml.foundation.core.MAssociationEnd;
import ru.novosoft.uml.foundation.core.MAssociationEndImpl;
import ru.novosoft.uml.foundation.core.MAttribute;
import ru.novosoft.uml.foundation.core.MAttributeImpl;
import ru.novosoft.uml.foundation.core.MClassifier;
import ru.novosoft.uml.foundation.core.MClassifierImpl;
import ru.novosoft.uml.foundation.core.MOperation;
import ru.novosoft.uml.foundation.core.MOperationImpl;
import VOCLModel.AssociationEndRole;
import VOCLModel.AssociationEndRoleSource;
import VOCLModel.AssociationEndRoleTarget;
import VOCLModel.Attribute;
import VOCLModel.BoolExp;
import VOCLModel.BoolType;
import VOCLModel.ClassifierNode;
import VOCLModel.ClassifierRole;
import VOCLModel.Condition;
import VOCLModel.Constraint;
import VOCLModel.Context;
import VOCLModel.IfExp;
import VOCLModel.Kind;
import VOCLModel.LetExp;
import VOCLModel.Method;
import VOCLModel.NavExpContainer;
import VOCLModel.OpKind;
import VOCLModel.PredicateContainer;
import VOCLModel.SetClassifierRole;
import VOCLModel.VOCLDiagramm;
import VOCLModel.VOCLExpression;
import VOCLModel.VariableDeclaration;
import de.tuberlin.tfs.ocl.metamodel.converter.NoSuchAttributeException;
import de.tuberlin.tfs.ocl.metamodel.expressions.MAssociationEndCallExp;
import de.tuberlin.tfs.ocl.metamodel.expressions.MAssociationEndCallExpImpl;
import de.tuberlin.tfs.ocl.metamodel.expressions.MAttributeCallExp;
import de.tuberlin.tfs.ocl.metamodel.expressions.MAttributeCallExpImpl;
import de.tuberlin.tfs.ocl.metamodel.expressions.MIfExp;
import de.tuberlin.tfs.ocl.metamodel.expressions.MIfExpImpl;
import de.tuberlin.tfs.ocl.metamodel.expressions.MIteratorExp;
import de.tuberlin.tfs.ocl.metamodel.expressions.MIteratorExpImpl;
import de.tuberlin.tfs.ocl.metamodel.expressions.MLetExp;
import de.tuberlin.tfs.ocl.metamodel.expressions.MLetExpImpl;
import de.tuberlin.tfs.ocl.metamodel.expressions.MLiteralExp;
import de.tuberlin.tfs.ocl.metamodel.expressions.MOclConstraint;
import de.tuberlin.tfs.ocl.metamodel.expressions.MOclConstraintImpl;
import de.tuberlin.tfs.ocl.metamodel.expressions.MOclExpression;
import de.tuberlin.tfs.ocl.metamodel.expressions.MOperationCallExp;
import de.tuberlin.tfs.ocl.metamodel.expressions.MOperationCallExpImpl;
import de.tuberlin.tfs.ocl.metamodel.expressions.MStringLiteralExp;
import de.tuberlin.tfs.ocl.metamodel.expressions.MStringLiteralExpImpl;
import de.tuberlin.tfs.ocl.metamodel.expressions.MVariableDeclaration;
import de.tuberlin.tfs.ocl.metamodel.expressions.MVariableDeclarationImpl;
import de.tuberlin.tfs.ocl.metamodel.expressions.MVariableExp;
import de.tuberlin.tfs.ocl.metamodel.expressions.MVariableExpImpl;
import de.tuberlin.tfs.ocl.metamodel.types.MOclConstraintKind;
import de.tuberlin.tfs.ocl.metamodel.types.MSetType;
import de.tuberlin.tfs.ocl.metamodel.types.MSetTypeImpl;

/**
 * This converter class was written for computing standard OCL strings out of an instance of the class VOCLDiagramm
 * which is part of the EMF model project VOCLModel3 and represents an VOCL (Visual OCL) diagram . It contains only one
 * public method that manages the conversion completely.
 * <p>
 * 
 * For more information about VOCL and OCL take a look at <em> www.tfs.cs.tu-berlin.de/vila </em>.
 * 
 * @author melodica (Tony Modica)
 *         <p>
 */

// TODO Die Variablenprfung ist sehr hinderlich und unflexibel
// und knnte als Hinweis gegeben werden? Aber nicht als Abbruchkriterium!
// Auerdem sollte man beliebige Conditions angeben knnen, 
// nicht nur durch binren Operator getrennt!

public class Converter {

	/**
	 * @param diag
	 *            An instance of the class VOCLDiagramm from VOCLModel3, generated and passed by the VOCLEditor3
	 * @return A textual string representing the diagram in OCL (Object Constraint Language)
	 */
	public static String convert(VOCLDiagramm diag) {
		// Erzeugt fuer jeden Constraint eine OCL-Metamodellinstanz
		// und verknuepft die daraus konvertierten OCL-Strings
		String result = "";

		if (diag.getConstraint().size() == 0) {
			return "Das Diagramm enthlt keine Constraints!";
		}

		for (Iterator i = diag.getConstraint().iterator(); i.hasNext();) {
			Constraint actConstraint = (Constraint) i.next();

			MOclConstraint actconstExp;

			try {
				actconstExp = getConstraintExpr(actConstraint);
				result = result + "\n\n" + actconstExp.convert2OclString();
			} catch (ConverterException e1) {
				return result + "\n\n" + e1.getMessage();
			} catch (NoSuchAttributeException e) {
				// System.out.println("Fehler! ConverterException in convert2OclString()");
				e.printStackTrace();
				return "Fehler! ConverterException in convert2OclString()";
			} catch (Exception e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}
		return result;
	}

	private static MOclConstraint getConstraintExpr(Constraint constr) throws ConverterException {
		// Context den Untermethoden zur Vefuegung stellen
		Context actContext = constr.getContext();

		// Contraint erzeugen, Name setzen
		MOclConstraint resultcon = new MOclConstraintImpl();
		resultcon.setName(constr.getName());

		// Context und Art des Constraints setzen
		switch (constr.getKind().getValue()) {
			case Kind.INV :
				resultcon.setKind(MOclConstraintKind.INV);

				break;

			case Kind.PRE :
				resultcon.setKind(MOclConstraintKind.PRE);

				break;

			case Kind.POST :
				resultcon.setKind(MOclConstraintKind.POST);

				break;

			default :
		}

		if (actContext.isIsClassifier()) {
			// Context ist ein Classifier
			MClassifier constrElem = new MClassifierImpl();
			constrElem.setName(constr.getContext().getName());
			resultcon.setConstrainedElement(constrElem);
		} else {
			// Context ist eine Methode
			// Typ der Operation
			MClassifier opType = new MClassifierImpl();
			opType.setName(actContext.getType());

			// Referenzierte Operation
			MOperation op = new MOperationImpl();
			op.setName(actContext.getName());

			// OperationCallExpression zusammensetzen und in Constraint haengen
			MOperationCallExp contextOpCallExp = new MOperationCallExpImpl();
			contextOpCallExp.setReferredOperation(op);
			contextOpCallExp.setType(opType);
			resultcon.setConstrainedElement(contextOpCallExp);

			// Argumente sammeln
			for (Iterator i = actContext.getVariableDeclaration().iterator(); i.hasNext();) {
				VariableDeclaration varDec = (VariableDeclaration) i.next();

				// Typ der Variablen
				MClassifier varType = new MClassifierImpl();
				varType.setName(varDec.getType());

				// Name der Variablen
				MAttribute attr = new MAttributeImpl();
				attr.setName(varDec.getVarName());
				attr.setType(varType);

				// AttributeCallExpr bauen
				MAttributeCallExp methodAttrCall = new MAttributeCallExpImpl();
				methodAttrCall.setType(varType);
				methodAttrCall.setReferredAttribute(attr);

				// und in die OpCallExpr einhaengen
				contextOpCallExp.addArguments(methodAttrCall);
			}
		}

		// OCLExpressions einsetzen
		ArrayList initList = new ArrayList();
		MOclExpression resultOCLExpr;

		resultOCLExpr = selectAndGetOCLExpr(constr.getVOCLExpression(), initList);

		resultcon.setBody(resultOCLExpr);

		return resultcon;
	}

	private static Constraint getConstraint(VOCLExpression e) {
		Constraint constr = null;
		VOCLExpression expr = e;

		while (constr == null) {
			if (expr instanceof Constraint) {
				constr = (Constraint) expr;
			} else {
				expr = (VOCLExpression) expr.eContainer();
			}
		}

		return constr;
	}

	/**
	 * @param expression
	 * @return
	 */

	private static MOclExpression selectAndGetOCLExpr(VOCLExpression expr, ArrayList condList)
			throws ConverterException {
		Constraint actConstraint = getConstraint(expr);
		Context actContext = actConstraint.getContext();

		// Der einfachste Fall: NavExpContainer, einfach Gleichungen erzeugen
		if (expr instanceof NavExpContainer) {
			MOclExpression resultExp = null;

			// Das try-catch verlangt der Compiler, PassConditionList tritt
			// nicht auf (wg. false)!
			try {
				resultExp = getNavConjExp((NavExpContainer) expr, condList, false);
			} catch (ConverterException e) {
				// Der Converter-Fehler kann ganz normal weiter hochgereicht
				// werden
				throw e;
			} catch (PassConditionListException e) {
				// Das kann gar nicht auftreten!
			}

			if (expr.isIsNegated()) {
				return createNegation(resultExp);
			}

			return resultExp;
		}

		if (expr instanceof LetExp) {
			// Nur bei NavExpContainer wird der Variablen ein Wert zugewiesen!
			// Bei anderen VOCL-Expr einfach die zugehoerige OCL-Expr der
			// Variablen
			// zuweisen. Ob das Sinn macht ist eine andere Frage...
			MLetExp resultExp = new MLetExpImpl();

			// Variable aus dem VOCL-Let-Audruck auspacken und deren Namen
			// zur Verfuegung stellen
			VariableDeclaration letVarDec = ((LetExp) expr).getVariableDeclaration();
			String varName = letVarDec.getVarName();

			// OCL-Variable erzeugen
			MVariableDeclaration oclVarDec = new MVariableDeclarationImpl();
			oclVarDec.setVarname(varName);

			// LET und IN-Ausdruecke stellen
			VOCLExpression letExp = ((LetExp) expr).getLetExpression();
			VOCLExpression inExp = ((LetExp) expr).getInExpression();

			ArrayList initList = new ArrayList(condList);

			// Ist die letExpr ein NavExpContainer?
			if (letExp instanceof NavExpContainer) {
				// Hier wird ganz uebel getrickst! Wir missbrauchen
				// getNavExpConjExp
				// um die Condition-Liste zu erweitern, und holen diese aus der
				// erzwungenen Condition! Das Argument returnCondList gibt an,
				// ob
				// die Condition-Liste als Exception weitergeworfen werden muss.
				// Dann suchen wir uns aus der Liste den Ausdruck zur Condition.
				// Die Let-Variable muss zunaechst in der Conditionliste
				// deklariert werden.
				// Nach dem Auswerten mit getNavConjExp wird die richtige
				// ConditionInfo erzeugt werden... hoffentlich
				// Dazu wird eine Dummygleichung eingefuegt
				ConditionInfo dummyCInfo = new ConditionInfo(varName, "=", varName, letVarDec.getType(), true);
				initList.add(dummyCInfo);

				// Einen Zeiger fuer die erweiterte Conditionlist initialisieren
				ArrayList extCondList = new ArrayList();

				try {
					// Die Conditionliste soll hochgereicht werden (true)
					getNavConjExp((NavExpContainer) letExp, initList, true);
				} catch (ConverterException e) {
					// Ein herkoemmlicher Converter-Fehler ist aufgetreten
					// einfach weiterreichen...
					throw e;
				} catch (PassConditionListException e) {
					// Eine evntl. erweiterte ConditionList wurde per Exception
					// hochgeworfen. Auspacken und zum Auswerten weiter geben!
					// Dabei wird die Dummygleichung wieder entfernt, damit die
					// Expression, die der Let-Variablen zugewiesen wurde
					// nicht aus der Dummy-Gleichung kommt!
					extCondList = e.getCondList();
					extCondList.remove(dummyCInfo);
				}

				// jetzt noch die Variable in der extCondList suchen,
				// dabei auf doppelte und fehlende Definition pruefen
				MOclExpression initExp = null;

				for (Iterator i = extCondList.iterator(); i.hasNext();) {
					ConditionInfo cInfo = (ConditionInfo) i.next();

					// Gab der Iterator eine Variable passenden Namens?
					if (cInfo.getVariable().equals(varName)) {
						// Wurde schon vorher eine Variable gefunden? Fehler!
						if (initExp != null && cInfo.getOpCall() instanceof MStringLiteralExp
								&& !cInfo.getOpCall().getName().equals(initExp.getName())) {
							throw new ConverterException("Im Constraint '" + actConstraint.getName()
									+ "' mit dem Context '" + actContext.getName() + "' kann der \n LET-Variablen '"
									+ varName + "' nicht eindeutig ein Ausdruck\nzugewiesen werden,"
									+ " da diese mehrfach verwendet wird!");

						}
						// sonst den Wert aus der Conditionliste holen
						if (cInfo.getOpCall() != null) {
							initExp = cInfo.getOpCall();
						} else {
							initExp = new MStringLiteralExpImpl();
							initExp.setName(cInfo.getAssociated());
							MClassifier strExpType = new MClassifierImpl();
							strExpType.setName(cInfo.getType());
							initExp.setType(strExpType);

						}
					}
				}

				// Wurde eine passende Condition gefunden?
				if (initExp == null) {
					throw new ConverterException("Im Constraint '" + actConstraint.getName() + "' mit dem Context '"
							+ actContext.getName() + "' kann der \nLET-Variablen '" + varName
							+ "' nicht eindeutig ein Ausdruck zugewiesen werden,\n" + "da diese nicht verwendet wird!");
				}

				oclVarDec.setInitExpression(initExp);

				// Variablentyp erzeugen und vergeben
				MClassifier varType = new MClassifierImpl();
				varType.setName(initExp.getType().getName());
				oclVarDec.setType(varType);

				// In der Condition-Liste die Variablendefinition umstellen auf
				// !isVar
				// damit die In-Expression die Variable vorfindet
				dummyCInfo.setAssIsVar(false);
			} else {
				// Wenn kein NavExpContainer vorhanden ist, dann einfach die
				// letExp in OCL
				// umwandeln und der Variablen zuweisen
				MOclExpression oclLetExp = selectAndGetOCLExpr(letExp, condList);
				oclVarDec.setInitExpression(oclLetExp);

				// Variablentyp erzeugen und vergeben
				MClassifier varType = new MClassifierImpl();
				varType.setName(oclLetExp.getType().getName());
				oclVarDec.setType(varType);

				// Auch hier Dummygleichung einfuegen, damit der IN-Teil die
				// Variable findet
				ConditionInfo dummyCInfo = new ConditionInfo(varName, "=", varName, letVarDec.getType(), false);
				initList.add(dummyCInfo);
			}

			// Test auf Typengleichheit zwischen der Variablen und der InExp
			if (!oclVarDec.getType().getName().equals(letVarDec.getType())) {
				// Die Typen sind ungleich! Fehler!
				throw new ConverterException("Im Constraint '" + actConstraint.getName() + "' mit dem Context '"
						+ actContext.getName() + "'\n wird der LET-Variablen '" + varName + "' des Typs '"
						+ letVarDec.getType() + "'\n" + "ein Ausdruck des Typs '" + oclVarDec.getType().getName()
						+ "' zugewiesen!");
			}

			// Wenn der Typ stimmt, die OCL-Variable in die resultExp einhaengen
			resultExp.setVariable(oclVarDec);

			// Nun noch die inExp auswerten und in die resultExp einhaengen
			MOclExpression oclInExp = selectAndGetOCLExpr(inExp, initList);
			resultExp.setIn(oclInExp);
			resultExp.setType(oclInExp.getType());

			return resultExp;
		}

		if (expr instanceof IfExp) {
			MIfExp resultExp = new MIfExpImpl();
			IfExp ifExpr = (IfExp) expr;
			resultExp.setCondition(selectAndGetOCLExpr(ifExpr.getIfExpression(), condList));
			resultExp.setTheExpression(selectAndGetOCLExpr(ifExpr.getThenExpression(), condList));
			resultExp.setElseExpression(selectAndGetOCLExpr(ifExpr.getElseExpression(), condList));

			// Der Typ sollte hier eigentlich IMMER Boolean sein!
			resultExp.setType(resultExp.getTheExpression().getType());

			return resultExp;
		}

		if (expr instanceof BoolExp) {
			BoolExp boolExp = (BoolExp) expr;
			MOclExpression arg1 = selectAndGetOCLExpr(boolExp.getOp1(), condList);
			MOclExpression arg2 = selectAndGetOCLExpr(boolExp.getOp2(), condList);

			// Bool-Operator bestimmen
			String actOp = "!Operation unbekannt!";

			switch (boolExp.getKind().getValue()) {
				case BoolType.OR :
					actOp = "or";

					break;

				case BoolType.AND :
					actOp = "and";

					break;

				case BoolType.XOR :
					actOp = "xor";

					break;

				case BoolType.IMPLIES :
					actOp = "implies";

					break;
			}

			MOperationCallExp resultExp = createEqn(arg1, arg2, actOp);

			if (expr.isIsNegated()) {
				return createNegation(resultExp);
			}

			return resultExp;
		}

		// Falls wir hier ankommen, ist der VOCLExpressiontyp unbekannt!
		throw new ConverterException("Im Constraint '" + actConstraint.getName() + "' mit dem Context '"
				+ actContext.getName() + "'\nfehlt ein VOCL-Ausdruck bzw. dessen Container!");
	}

	private static MOclExpression createNegation(MOclExpression exp) {
		// leere Expressions ignorieren, dafuer ist schon ein Fehler erzeugt
		// worden!
		if (exp == null)
			return null;

		// Booleantyp erzeugen
		MClassifier booleanType = new MClassifierImpl();
		booleanType.setName("Boolean");

		// Negation erzeugen
		MOperationCallExp neg = new MOperationCallExpImpl();
		neg.setType(booleanType);

		//Operation op an Gleichung binden
		MOperation eqnOperation = new MOperationImpl();
		eqnOperation.setName("not");
		neg.setReferredOperation(eqnOperation);

		// Argument in die Negation einfuegen
		neg.addArguments(exp);
		exp.setParentOperation(neg);

		return neg;
	}

	private static ArrayList extractClassifierRoles(EList classifierNodes) {
		ArrayList classifierRoles = new ArrayList();

		for (Iterator i = classifierNodes.iterator(); i.hasNext();) {
			ClassifierNode actNode = (ClassifierNode) i.next();
			if (actNode instanceof ClassifierRole) {
				classifierRoles.add(actNode);
			}
		}

		return classifierRoles;
	}

	private static MOclExpression getNavConjExp(NavExpContainer container, ArrayList initList, boolean returnCondList)
			throws ConverterException, PassConditionListException {
		Constraint actConstraint = getConstraint(container);
		Context actContext = actConstraint.getContext();

		// Liegt dieser Container in einem Predikatcontainer?
		PredicateContainer actPredContainer = null;
		VOCLExpression exp = container;
		while (actPredContainer == null && !(exp instanceof Constraint)) {
			if (exp instanceof PredicateContainer)
				actPredContainer = (PredicateContainer) exp;
			else
				exp = (VOCLExpression) exp.eContainer();
		}

		String predStr = "";
		if (actPredContainer != null) {
			predStr = "Prdikat-Container im ";
		}

		// Classifierroles auspacken und zur Verfuegung stellen
		ArrayList classifierRoles = extractClassifierRoles(container.getClassifierNode());

		ArrayList navEquations = new ArrayList();

		// Variablenliste aus der Condition aufbauen
		ArrayList condList = getConditionVarList(container.getCondition());

		// Hier werden die Conditions berprft, ob sie schon Gleichungen
		// erzeugen, da auch ein leerer NavExpContainer in einem In-Teil mit den
		// entsprechenden Conditions Gleichungen erzeugen kann!
		// initList danach noch fuer das normale Traversieren in die neue
		// condList einfuegen!
		ArrayList condListWorkingCopy = new ArrayList(condList);

		// TODO Testen, ob ein Update von condList hierbei notwendig ist!
		// Ich glaube nicht, da die Variablen, die nur in der Condition
		// vorkommen, ja "gespiegelt" wurden.
		for (Iterator i = condList.iterator(); i.hasNext();) {
			ConditionInfo actCInfo = (ConditionInfo) i.next();
			String actVarName = actCInfo.getVariable();
			String actVarType = actCInfo.getType();

			// Durchsuche jede der uebergebenen ConditionInfos, ob die
			// aktuelle CInfo-Variable vorkommt.
			for (Iterator j = initList.iterator(); j.hasNext();) {
				ConditionInfo testCInfo = (ConditionInfo) j.next();
				String testVarName = testCInfo.getVariable();
				String testVarType = testCInfo.getType();

				if (actVarName.equals(testVarName)) {
					if (!actVarType.equals(testVarType)) {
						throw new ConverterException("Im " + predStr + "Constraint '" + actConstraint.getName()
								+ "' mit dem Context '" + actContext.getName() + "' ist wurde \ndie LET-Variable '"
								+ actVarName + "' als '" + testVarType + "' deklariert,\n" + "jedoch als '"
								+ actVarType + "' verwendet!");
					}

					// OCL-Variable erzeugen mitgeben
					// VariableExpression aufbauen
					MVariableExp varExp = new MVariableExpImpl();
					MVariableDeclaration varDec = new MVariableDeclarationImpl();

					// VariableExpression erzeugen
					MClassifier varType = new MClassifierImpl();
					varType.setName(actVarType);
					varExp.setType(varType);
					varDec.setType(varType);
					varExp.setReferredVariable(varDec);
					varDec.setVarname(actVarName);

					navEquations.addAll(getSolvedExpXorUpdateList(actVarName, condListWorkingCopy, varExp, actContext));
				}
			}
		}

		// Wenn Gleichungen erzeugt wurden aber keine ClassifierRoles vorhanden sind,
		// dann knnen hier schon die Gleichungen zurueckgegeben werden
		if ((navEquations.size() > 0) && (classifierRoles.size() == 0) && !returnCondList) {
			return conjugateEqns(navEquations);
		} else
			if ((navEquations.size() > 0) && (classifierRoles.size() == 0) && returnCondList) {
				throw new PassConditionListException(condList);
			}

		condList.addAll(initList);

		// Der Fall dass keine ClassifierNodes existieren, kann hier abgefangen werden,
		// damit in Methoden-Constraints keine NullpointerExceptions auftreten.
		// Aber: Wenn noch Variablen in einem Methoden-Context stehen und in einer
		// Condition erwhnt werden knnen auch so Gleichungen erzeugt werden,
		// also keine Exception werfen
		int actKind = actConstraint.getKind().getValue();
		if (container.getClassifierNode().size() == 0
				&& (((actKind != Kind.POST) && (actKind != Kind.PRE)) || (actContext.getVariableDeclaration().size() == 0))) {
			// Das Modell enthlt keine ClassifierNodes
			throw new ConverterException("Der " + predStr + "Constraint '" + actConstraint.getName()
					+ "' mit dem Context '" + actContext.getName()
					+ "'\nenthaelt keine ClassifierRoles oder Collections\n" + "und erzeugt somit keine Gleichungen!");
		}

		ClassifierNode start = null;
		if (actPredContainer == null) {
			// normal Startpunkt finden
			start = findStartClassifierNode(container);
		} else {
			// Die Start-ClassifierRole ist diejenige, die mit dem Namen der Iteratorvariable bereinstimmt.
			VariableDeclaration actIteratorVar = actPredContainer.getIterator();

			for (Iterator i = classifierRoles.iterator(); i.hasNext();) {
				ClassifierNode actClNode = (ClassifierNode) i.next();
				if (actClNode instanceof ClassifierRole && actClNode.getCallVar() != null
						&& actClNode.getCallVar().getVarName() == actIteratorVar.getVarName()) {
					if (start == null) {
						start = (ClassifierRole) actClNode;
					} else {
						throw new ConverterException("Der " + predStr + "Constraint '" + actConstraint.getName()
								+ "' mit dem Context '" + actContext.getName()
								+ "'\nenthaelt mehrere ClassifierRoles mit dem Namen des Iterators '"
								+ actIteratorVar.getVarName() + "',"
								+ "\nso dass kein eindeutiger Startpunkt fr die Navigationen gefunden werden kann!");
					}
				}
			}

			// wurde ein Startpunkt im Prdikatcontainer gefunden?
			if (start == null) {
				throw new ConverterException("Der " + predStr + "Constraint '" + actConstraint.getName()
						+ "' mit dem Context '" + actContext.getName()
						+ "'\nenthaelt keine ClassifierRole mit dem Namen des Iterators '"
						+ actIteratorVar.getVarName() + "',"
						+ "\nso dass kein eindeutiger Startpunkt fr die Navigationen gefunden werden kann!");
			}
		}

		// und Navigationsgleichungen zusammenstellen
		navEquations.addAll(getNavEqns(condList, actContext, start));

		// Da der Context fuer den LET-Fall keine Bedeutung hat, kann
		// hier schon die Coditionlist geworfen werden, falls erforderlich
		if (returnCondList) {
			throw new PassConditionListException(condList);
		}

		// Methoden-Variablen auf Gleichungen mit der Condition testen
		if ((actConstraint.getKind().getValue() == Kind.PRE) || (actConstraint.getKind().getValue() == Kind.POST)) {
			for (Iterator i = actContext.getVariableDeclaration().iterator(); i.hasNext();) {
				VariableDeclaration actPar = (VariableDeclaration) i.next();

				// VariableExpression aufbauen
				MVariableExp varExp = new MVariableExpImpl();
				MVariableDeclaration varDec = new MVariableDeclarationImpl();

				// VariableExpression erzeugen
				MClassifier varType = new MClassifierImpl();
				varType.setName(actPar.getType());
				varExp.setType(varType);
				varDec.setType(varType);
				varExp.setReferredVariable(varDec);
				varDec.setVarname(actPar.getVarName());

				// Variable auflsen und Gleichung hinzufgen
				navEquations.addAll(getSolvedExpXorUpdateList(actPar.getVarName(), condList, varExp, actContext));
			}
		}

		// Spezialfall: Gleichungen auch fr self-Variable erzeugen!!!
		// VariableExpression aufbauen
		MVariableExp varExp = new MVariableExpImpl();
		MVariableDeclaration varDec = new MVariableDeclarationImpl();

		// VariableExpression erzeugen
		MClassifier varType = new MClassifierImpl();
		varType.setName(start.getType());
		varExp.setType(varType);
		varDec.setType(varType);
		varExp.setReferredVariable(varDec);
		varDec.setVarname("self");

		// Variable auflsen und Gleichung hinzufgen
		navEquations.addAll(getSolvedExpXorUpdateList("self", condList, varExp, actContext));
		
		// Spezialfall: Gleichungen fr "true" erzeugen
		
		// VariableExpression aufbauen
		varExp = new MVariableExpImpl();
		varDec = new MVariableDeclarationImpl();

		// VariableExpression erzeugen
		 varType = new MClassifierImpl();
		varType.setName("Boolean");
		varExp.setType(varType);
		varDec.setType(varType);
		varExp.setReferredVariable(varDec);
		varDec.setVarname("true");

		// Variable auflsen und Gleichung hinzufgen
		navEquations.addAll(getSolvedExpXorUpdateList("true", condList, varExp, actContext));
		
		if (navEquations.size() == 0) {
			// Das Modell erzeugt keine Gleichung
			throw new ConverterException("Der " + predStr + "Constraint '" + actConstraint.getName()
					+ "' mit dem Context '" + actContext.getName() + "'\nerzeugt keine Gleichungen!");
		}

		// die Gleichungen muessen konjugiert werden, bzw. eine Gleichung wird eingefuegt
		return conjugateEqns(navEquations);

	}

	private static MOclExpression conjugateEqns(ArrayList navEquations) {
		if (navEquations.size() == 1) {
			return (MOclExpression) navEquations.get(0);
		}
		// 1. Konjunktion in Constraint einfuegen
		MOperationCallExp actualConj = createConj();

		// Wurzelzeiger
		MOperationCallExp completeConj = actualConj;
		
		// Alle Gleichungen nacheinander konjugieren
		Iterator i = navEquations.iterator();
		MOclExpression actualConjExp = (MOclExpression) i.next();
		actualConj.addArguments(actualConjExp);

		while (i.hasNext()) {
			actualConjExp = (MOperationCallExp) i.next();

			if (i.hasNext()) {
				MOperationCallExp newConj = createConj();

				actualConj.addArguments(newConj);
				actualConj = newConj;

				// neue Gleichung einfuegen
				actualConj.addArguments(actualConjExp);
			} else {
				// letzte Gleichung einfuegen
				actualConj.addArguments(actualConjExp);
			}	
		}
		
		return completeConj;

	}

	private static ArrayList getConditionVarList(EList conditions) {
		ArrayList condList = new ArrayList();

		if (conditions == null) {
			return condList;
		}

		for (Iterator i = conditions.iterator(); i.hasNext();) {
			Condition actCond = (Condition) i.next();
			String actVar = actCond.getLeft().getVarName();
			String actOp = actCond.getOp();
			String actType = actCond.getLeft().getType();

			// Doppelte Variablendefinitionen duerfen vorkommen,
			// werden, falls Typunterschiede bestehen beim
			// Erzeugen der "Gleichungen" abgefangen, vielleicht auch nicht?
			// Ist der rechte Wert der Condition auch eine Variable?
			if (actCond.getRight() == null) {
				String actVal = actCond.getValue();
				condList.add(new ConditionInfo(actVar, actOp, actVal, actType, false));
			} else {
				String actAss = actCond.getRight().getVarName();
				condList.add(new ConditionInfo(actVar, actOp, actAss, actType, true));

				// Wenn zwei Variablen in einer Condition auftauchen
				// wird die Condition auch "gespiegelt" in die Liste aufgenommen,
				// damit die Reihenfolge des Auffindens der Variablen waehrend der
				// Graphtraversierung keine Rolle spielt.
				String actInvOp = "";

				if (actOp.equals("=")) {
					actInvOp = "=";
				} else
					if (actOp.equals("<=")) {
						actInvOp = ">=";
					} else
						if (actOp.equals("<")) {
							actInvOp = ">";
						} else
							if (actOp.equals("=>")) {
								actInvOp = "<=";
							} else
								if (actOp.equals(">")) {
									actInvOp = "<";
								}

				condList.add(new ConditionInfo(actAss, actInvOp, actVar, actType, true));
			}
		}

		return condList;
	}

	private static ClassifierNode findStartClassifierNode(NavExpContainer container) throws ConverterException {
		ClassifierNode startNode = null;

		Constraint actConstraint = getConstraint(container);
		Context actContext = actConstraint.getContext();

		// error enthaelt die aktuelle Fehlerursache
		String error;

		if (actContext.isIsClassifier()) {
			// Es kann eine Variable gesetzt sein, oder nicht (= "")
			VariableDeclaration varDec = (VariableDeclaration) actContext.getVariableDeclaration().get(0);
			String varName = varDec.getVarName();
			String varType = varDec.getType();

			if (!varName.equals("")) {
				// Jetzt muss eine ClassifierRole mit der Variable
				// varName (eventuell auch self) existieren
				startNode = forceRoleWithVarWithoutDubs(varName, varType, container);
				error = "Im Constraint '" + actConstraint.getName() + "' mit dem Context '" + actContext.getName()
						+ "'\nkann die Ausgangs-ClassifierRole der Navigationen\n"
						+ "nicht eindeutig bestimmt werden, da keine ClassifierRole\n" + "die Variablenbezeichnung '"
						+ varName + "' enthaelt!";
			} else {
				// Dann muss eben eine ClassifierRole mit self existieren
				startNode = forceRoleWithVarWithoutDubs("self", varType, container);
				error = "Im Constraint '" + actConstraint.getName() + "' mit dem Context '" + actContext.getName()
						+ "'\nkann die Ausgangs-ClassifierRole der Navigationen\n"
						+ "nicht eindeutig bestimmt werden, da keine ClassifierRole\n"
						+ "die Variablenbezeichnung 'self' enthaelt und der Context\n"
						+ "keine alternative Start-ClassifierRole bestimmt!";
			}

			if (startNode == null) {
				// Spezialfall: Wenn genau eine ClassifierCollection mit Namen (oder "self") und Typ des Context
				// existiert und
				// zustzlich daraus nur ein Prdikatcontainer annavigierbar ist, wird die Collection als Startknoten
				// genommen und spter allInstances() erzeugt!
				if (varName.equals(""))
					varName = "self";
				startNode = startPredicateCollection(container);
			}

		} else {
			// Bei Methodenaufrufen muss eine "self"-ClassifierRole existieren,
			// da kein alternativer Startpunkt definiert werden kann.
			// Sonst gaebe es keine Navigation
			// Mit einem Stringtokenizer den Klassentyp der Methode herausfinden:
			// (erstes Token bis zu einem ":")
			StringTokenizer st = new StringTokenizer(actContext.getName(), ":");
			String varType = st.nextToken();
			startNode = forceRoleWithVarWithoutDubs("self", varType, container);
			error = "Im Constraint '" + actConstraint.getName() + "' mit dem Context '" + actContext.getName()
					+ "'\nkann die Ausgangs-ClassifierRole der Navigationen\n"
					+ "nicht eindeutig bestimmt werden, da keine ClassifierRole\n"
					+ "die Variablenbezeichnung 'self' enthaelt!";
		}

		// sonst Fehler ausgeben, wenn kein StartNode gefunden wurde
		if (startNode == null) {
			throw new ConverterException(error);
		}

		return startNode;
	}

	private static SetClassifierRole startPredicateCollection(NavExpContainer container)
			throws ConverterException {
		SetClassifierRole startNode = null;

		for (Iterator i = container.getClassifierNode().iterator(); i.hasNext();) {
			ClassifierNode actNode = (ClassifierNode) i.next();
			// Bedingungen: Genau eine SetClassifierRole, ohne navigierbare Assoziationen
			// Wenn kein Predikatcontainer an dieser hngt, wird sowieso keine Gleichung erzeugt
			if (actNode instanceof SetClassifierRole && navableAssociationEndRoles(actNode).size() == 0) {
				if (startNode == null) {
					startNode = (SetClassifierRole) actNode;
				} else {
					Constraint actConstraint = getConstraint(container);
					Context actContext = actConstraint.getContext();
					// Es gibt mehrere passende SetClassifierRoles!
					throw new ConverterException("Im Constraint '" + actConstraint.getName() + "' mit dem Context '"
							+ actContext.getName() + "'\nkann die Ausgangs-SetClassifierRole der Navigationen"
							+ "\nnicht eindeutig bestimmt werden, da mehrere SetClassifier"
							+ "\nund keine 'self'-ClassifierRole vorhanden ist!");
				}
			}
		}

		return startNode;
	}

	private static ClassifierRole forceRoleWithVarWithoutDubs(String contextName, String contextType,
			NavExpContainer container) throws ConverterException {
		Constraint actConstraint = getConstraint(container);
		Context actContext = actConstraint.getContext();

		ClassifierRole startRole = null;

		for (Iterator i = container.getClassifierNode().iterator(); i.hasNext();) {
			ClassifierNode cNode = (ClassifierNode) i.next();

			if (cNode instanceof ClassifierRole) {

				String actVarName = "";
				String actVarType = ((ClassifierRole) cNode).getType();
				if (cNode.getCallVar() != null) {
					VariableDeclaration actVar = cNode.getCallVar();
					actVarName = actVar.getVarName();
				}

				if (actVarName.equals(contextName)) {
					if (startRole != null) {
						// es gibt zwei ClassifierRoles mit contextName!
						throw new ConverterException("Im Constraint '" + actConstraint.getName()
								+ "' mit dem Context '" + actContext.getName()
								+ "'\nkann die Ausgangs-ClassifierRole der Navigationen\n"
								+ "nicht eindeutig bestimmt werden, da die Variablenbezeichnung \n'" + contextName
								+ "' mehrfach vergeben wurde!");
					} else
						if (contextType != null && !contextType.equals(actVarType)) {
							// Stimmt der Type der gefundenen
							// StartclassifierRole
							// auch
							// mit dem Context berein?
							throw new ConverterException("Im Constraint '" + actConstraint.getName()
									+ "' mit dem Context '" + actContext.getName()
									+ "'\nstimmt der Typ der ClassifierRole '" + actVarName + ":" + actVarType + "'\n"
									+ "nicht mit dem des Context berein!");
						} else {
							// ClassifierRole merken
							startRole = (ClassifierRole) cNode;
						}
				}
			}
		}

		// ist eventuell null, wenn keine passende ClassifierRole gefunden
		// wurde!
		return startRole;
	}

	private static ArrayList getNavEqns(ArrayList condList, Context actContext, ClassifierNode start)
			throws ConverterException {
		ArrayList navEqns = new ArrayList();

		// In dieser Liste stehen die moeglichen Pfade von self aus
		ArrayList paths = new ArrayList();
		ArrayList allPaths = new ArrayList();

		// Startpfad erzeugen und in die Pfadliste einfuegen
		GraphPath newPath = new GraphPath(start);
		paths.add(newPath);
		allPaths.add(newPath);

		while (paths.size() != 0) {
			GraphPath actPath;

			// Nehme den ersten Pfad aus der Liste
			actPath = (GraphPath) paths.get(0);
			paths.remove(actPath);

			// Aktueller ClassifierNode
			ClassifierNode actNode = actPath.getActual();

			// Gleichung fuer einen Methodenaufruf erzeugen, falls einer da ist
			if (actNode instanceof ClassifierRole && (((ClassifierRole) actNode).getMethod() != null)) {
				navEqns.addAll(createMethodEqns((ClassifierRole) actNode, actPath, condList, actContext));
			}

			// Collectionoperationen auswerten
			if (actNode instanceof SetClassifierRole
					&& (((SetClassifierRole) actNode).getOpKind().getValue() != OpKind.NO_OP)) {
				navEqns.addAll(createCollectionEqns((SetClassifierRole) actNode, actPath, condList, actContext));
			}

			// Navigierbare AssociationEndRoles des akt. Pfades finden
			ArrayList navableAssEndRoles = navableAssociationEndRoles(actNode);

			// Moegliche Pfade erzeugen, dabei auf Navigationssenke pruefen und Attributnav.-Gleichung erzeugen
			EList actAttributes = actNode.getAttribute();

			if (navableAssEndRoles.size() == 0 && (actAttributes.size() > 0)) {
				navEqns.addAll(createAttribExp(actPath, actAttributes, condList, actContext));
			} else {

				if (navableAssEndRoles.size() == 0 && actNode instanceof SetClassifierRole
						&& ((SetClassifierRole) actNode).getPredicateContainer() != null) {
					navEqns.add(createPredExp((SetClassifierRole) actNode, actPath, condList));
				}

				// jeden moeglichen Pfad erweitern und pruefen
				for (Iterator i = navableAssEndRoles.iterator(); i.hasNext();) {

					// FIXME hier ist was durcheinander gekommen ?
					newPath = new GraphPath(actPath);

					// Achtung, der neue EndClassifierNode kann auch eine Collection sein!
					AssociationEndRole actualEndRole = (AssociationEndRole) i.next();
					newPath.setActual(actualEndRole.getType());
					newPath.addLink(actualEndRole);

					// Wenn ein Pfad in sich selbst zurckfhrt, dann wird er nicht weiter behandelt,
					// da er keine Navigation erzeugt
					if (newPath.isCycle()) {
						allPaths.add(newPath);
						continue;
					}

					// auf aequivalente Wege pruefen und Gleichungen erzeugen
					// Suche Pfade in der Liste, in die der aktuelle Pfad muendet
					/*
					 * System.out.print("all: "); show(allPaths); System.out.print("act: "); show(paths);
					 * newPath.show(); System.out.print("\n");
					 */

					ArrayList equivPaths = newPath.pathsIncludingActual(allPaths);
					if (equivPaths.size() != 0) {
						navEqns.addAll(createEquivExp(newPath, equivPaths, navEqns));
						allPaths.add(newPath);
					}

					// - auf Kreislauf pruefen, dann auch Gleichung erzeugen
					if (newPath.isSelfCycle()) {
						navEqns.add(createSelfCycleExp(newPath));
						allPaths.add(newPath);
					}

					// und falls kein Super-Pfad in der Liste vorhanden ist, dort einfuegen
					else {
						// FIXME msste hier nicht auch allPaths stehen? Beispiel finden!
						if (!containSuperPath(paths, newPath)) {
							paths.add(newPath);
							allPaths.add(newPath);
						}
					}
				}
			}
		}

		return navEqns;
	}

	private static MIteratorExp createPredExp(SetClassifierRole actNode, GraphPath actPath, ArrayList condList)
			throws ConverterException {
		PredicateContainer predCont = actNode.getPredicateContainer();

		MClassifier booleanType = new MClassifierImpl();
		booleanType.setName("Boolean");
		MClassifier iterVarType = new MClassifierImpl();
		iterVarType.setName(predCont.getIterator().getType());

		MVariableDeclaration iterVarDecl = new MVariableDeclarationImpl();
		iterVarDecl.setType(iterVarType);
		iterVarDecl.setVarname(predCont.getIterator().getVarName());

		/*
		 * zur Orientierung Person.allInstances()->forall(p1: Person | forall(p2: Person | p1 <> p2 implies p1.name <>
		 * p2.name) // Iterator exp: forall(p2 | ...)
		 * 
		 * MIteratorExp p2IteratorExp = new MIteratorExpImpl(); p2IteratorExp.setType(booleanType);
		 * p2IteratorExp.setName("forall"); p2IteratorExp.setBody(impliesOpCallExp);
		 * p2IteratorExp.setIterator(p2VarDecl); // Iterator exp: forall(p1 | forall (p2 | ...) ) MIteratorExp
		 * p1IteratorExp = new MIteratorExpImpl(); p1IteratorExp.setType(booleanType); p1IteratorExp.setName("forall");
		 * p1IteratorExp.setBody(p2IteratorExp); p1IteratorExp.setIterator(p1VarDecl);
		 * p1IteratorExp.setSource(allInstOpCallExp); // --- OclConstraint MOclConstraint constraint = new
		 * MOclConstraintImpl(); constraint.setKind(MOclConstraintKind.INV);
		 * constraint.setConstrainedElement(personType); constraint.setBody(p1IteratorExp); Wichtig! Der Iterator ist
		 * das Wurzelelement, nicht die Collection!
		 *  
		 */

		MIteratorExp iterExp = new MIteratorExpImpl();
		iterExp.setType(booleanType);
		iterExp.setName(predCont.getPredType().getName().toLowerCase());
		iterExp.setIterator(iterVarDecl);

		iterExp.setSource(createAssEndCallExpTree(actPath));
		iterExp.getSource().setType(iterVarType);

		condList.add(new ConditionInfo(predCont.getIterator().getVarName(), "=", predCont.getIterator().getVarName(), predCont.getIterator().getType(), true));
		
		// Hier wird die iterExp gebaut
		iterExp.setBody(selectAndGetOCLExpr(predCont.getVOCLExpression(), condList));

		return iterExp;
	}

	private static ArrayList createMethodEqns(ClassifierRole actRole, GraphPath actPath, ArrayList condList,
			Context actContext) throws ConverterException {
		Constraint actConstraint = actContext.getConstraint();

		ArrayList result = new ArrayList();

		Method actMethod = actRole.getMethod();
		VariableDeclaration rReturn = actMethod.getRReturn();

		// Wenn keine Rueckgabe-Variable definiert ist, ist Standard "result"!
		if (rReturn.getVarName().equals("")) {
			// Der Methodenaufruf mit "result" muss auf den StartClassifier
			// gehen!
			if (actPath.getActual() != actPath.getSource()) {
				throw new ConverterException("Im Constraint '" + actConstraint.getName() + "' mit dem Context '"
						+ actContext.getName() + "'\nwurde keine Variable fuer das Ergebnis der Methode\n" + "'"
						+ actMethod.getName() + "' definiert und die ClassifierRole,\n"
						+ "auf welche die Methode zugreift, traegt nicht den Namen 'self'."
						+ "Es kann keine Navigation fuer diesen Methodenaufruf gefunden werden!");
			}
			String actVarName = rReturn.getVarName();

			// Typ erzeugen
			MClassifier sourceType = new MClassifierImpl();
			sourceType.setName(actMethod.getType());

			// Navigationsbaum ueber die AssociationEndRoles aufbauen
			MOclExpression assEndCallExp = createAssEndCallExpTree(actPath);
			assEndCallExp.setType(sourceType);

			// Operationsaufruf erzeugen, vorbereiten
			MOperationCallExp opCallExp = new MOperationCallExpImpl();
			opCallExp.setType(sourceType);
			opCallExp.setSource(assEndCallExp);

			// Referred Operation vorbereiten
			MOperation op = new MOperationImpl();
			op.setName(actMethod.getName());
			opCallExp.setReferredOperation(op);
			opCallExp.setName(actMethod.getName());

			// Parameter einsetzen
			for (Iterator i = actMethod.getCall().iterator(); i.hasNext();) {
				VariableDeclaration actPar = (VariableDeclaration) i.next();

				// VariableExpression erzeugen
				MVariableExp actVar = new MVariableExpImpl();
				MVariableDeclaration actVarDec = new MVariableDeclarationImpl();

				MClassifier parType = new MClassifierImpl();
				parType.setName(actPar.getType());
				actVar.setType(parType);
				actVarDec.setType(parType);
				actVar.setReferredVariable(actVarDec);
				actVarDec.setVarname(actVarName);

				// Variablen einhaengen
				opCallExp.addArguments(actVar);
			}

			result.addAll(getSolvedExpXorUpdateList("result", condList, opCallExp, actContext));

		} else {
			String actVarName = rReturn.getVarName();

			// Ansonsten wird normal zum Methodenaufruf navigiert
			// weil eine Methode auf ein ex. Objekt ausgefuehrt werden muss
			// Typ-Classifier erzeugen
			MClassifier sourceType = new MClassifierImpl();
			sourceType.setName(actPath.getActual().getCallVar().getType());

			// Navigationsbaum ueber die AssociationEndRoles aufbauen
			MOclExpression assEndCallExp = createAssEndCallExpTree(actPath);
			assEndCallExp.setType(sourceType);

			// Typ erzeugen
			MClassifier type = new MClassifierImpl();
			type.setName(actMethod.getType());

			// Operationsaufruf erzeugen, vorbereiten und assEndCallExp
			// einhngen
			MOperationCallExp opCallExp = new MOperationCallExpImpl();
			opCallExp.setSource(assEndCallExp);
			opCallExp.setType(type);
			opCallExp.setName(actMethod.getName());

			// Referred Operation vorbereiten
			MOperation op = new MOperationImpl();
			op.setName(actMethod.getName());
			opCallExp.setReferredOperation(op);

			// Parameter einsetzen
			for (Iterator i = actMethod.getCall().iterator(); i.hasNext();) {
				VariableDeclaration actPar = (VariableDeclaration) i.next();

				// VariableExpression erzeugen
				MVariableExp actVar = new MVariableExpImpl();
				MVariableDeclaration actVarDec = new MVariableDeclarationImpl();
				MClassifier parType = new MClassifierImpl();
				parType.setName(actPar.getType());
				actVar.setType(parType);
				actVarDec.setType(parType);
				actVar.setReferredVariable(actVarDec);
				actVarDec.setVarname(actPar.getVarName());

				// Variable einhaengen
				opCallExp.addArguments(actVar);
			}

			result.addAll(getSolvedExpXorUpdateList(actVarName, condList, opCallExp, actContext));
		}

		return result;
	}

	/**
	 * @param actNode
	 * @param actPath
	 * @param condList
	 */
	private static ArrayList createCollectionEqns(SetClassifierRole actColl, GraphPath actPath, ArrayList condList,
			Context actContext) throws ConverterException {
		Constraint actConstraint = actContext.getConstraint();

		ArrayList result = new ArrayList();

		// SetTyp-Classifier erzeugen
		MSetType setType = new MSetTypeImpl();
		setType.setName(actPath.getActual().getCallVar().getType());

		// Navigationsbaum ueber die AssociationEndRoles aufbauen
		MOclExpression assEndCallExp = createAssEndCallExpTree(actPath);
		assEndCallExp.setType(setType);

		// Operationsaufruf erzeugen, vorbereiten und assEndCallExp einhaengen
		MOperationCallExp opCallExp = new MOperationCallExpImpl();

		// Referred Operation vorbereiten
		MOperation oclOp = new MOperationImpl();

		switch (actColl.getOpKind().getValue()) {
			case OpKind.COUNT :

				// -- Integertyp erzeugen
				MClassifier integerType = new MClassifierImpl();
				integerType.setName("Integer");
				opCallExp.setSource(assEndCallExp);
				opCallExp.setType(integerType);
				oclOp.setName("size");
				opCallExp.setReferredOperation(oclOp);

				// Variable aufloesen
				result.addAll(getSolvedExpXorUpdateList(actColl.getOpVar().getVarName(), condList, opCallExp,
						actContext));

				break;

			case OpKind.EMPTY :

				// -- Booleantyp erzeugen
				MClassifier booleanType1 = new MClassifierImpl();
				booleanType1.setName("Boolean");
				opCallExp.setSource(assEndCallExp);
				opCallExp.setType(booleanType1);
				oclOp.setName("isEmpty");
				opCallExp.setReferredOperation(oclOp);
				result.add(opCallExp);

				break;

			case OpKind.NOT_EMPTY :

				// -- Booleantyp erzeugen
				MClassifier booleanType2 = new MClassifierImpl();
				booleanType2.setName("Boolean");
				opCallExp.setSource(assEndCallExp);
				opCallExp.setType(booleanType2);
				oclOp.setName("notEmpty");
				opCallExp.setReferredOperation(oclOp);
				result.add(opCallExp);

				break;

			case OpKind.SUM :

				// Gibt es ueberhaupt Attribute?
				if (actColl.getAttribute() == null) {
					// Es gibt gar keine Attribute! Fehler!
					throw new ConverterException("Im Constraint '" + actConstraint.getName() + "' mit dem Context '"
							+ actContext.getName() + "'\nwurde fuer die '" + actColl.getType() + "'-Collection '"
							+ actColl.getName() + "'\n" + "' eine SUM-Operation definiert,\n"
							+ "obwohl sie keine Attribute besitzt!");
				}

				// Einige Namen besorgen
				String iterVar = actColl.getIteratorVar().getVarName();
				String resVar = actColl.getOpVar().getVarName();

				Attribute iterAttr = null;

				// Zur Iteratorvariablen passendes Attribut dieser Collection
				// finden
				for (Iterator j = actColl.getAttribute().iterator(); j.hasNext();) {
					Attribute actAttr = (Attribute) j.next();

					// Hat das Attribut die passende Variable als Wert?
					if ((actAttr.getVariableDeclaration() != null)
							&& actAttr.getVariableDeclaration().getVarName().equals(iterVar)) {
						// Wurde vorher schon ein passendes Attribut gefunden?
						if (iterAttr != null) {
							throw new ConverterException("Im Constraint '" + actConstraint.getName()
									+ "' mit dem Context '" + actContext.getName() + "'\n existieren in der '"
									+ actColl.getType() + "'-Collection '" + actColl.getName()
									+ "zu der Summenvariablen '" + iterVar + "'\n"
									+ "mehrere Attribute mit der Summenvariablen als Wert!");
						}
						iterAttr = actAttr;

					}
				}

				// Wurde ein Attribut gefunden?
				if (iterAttr == null) {
					throw new ConverterException("Im Constraint '" + actConstraint.getName() + "' mit dem Context '"
							+ actContext.getName() + "'\n existiert in der '" + actColl.getType() + "'-Collection '"
							+ actColl.getName() + "zu der Summenvariablen '" + iterVar + "'\n"
							+ "kein Attribut mit dieser Variablen als Wert!");
				}

				// Jetzt ist ein passendes Attribut gefunden!
				// -- Summentyp erzeugen (gleicher Typ wie das Attribut)
				MSetType sumType = new MSetTypeImpl();
				sumType.setName(iterAttr.getVariableDeclaration().getType());

				// AttributeCallExp zusammenstellen, mit Attributname erweitern
				MAttributeCallExp attributeCallExp = new MAttributeCallExpImpl();
				attributeCallExp.setSource(assEndCallExp);
				assEndCallExp.setAppliedProperty(attributeCallExp);
				attributeCallExp.setType(sumType);

				MAttribute attribute = new MAttributeImpl();
				attribute.setName(iterAttr.getName());
				attributeCallExp.setReferredAttribute(attribute);

				opCallExp.setType(sumType);
				oclOp.setName("sum");
				opCallExp.setReferredOperation(oclOp);
				opCallExp.setSource(attributeCallExp);

				// Variable aufloesen
				result.addAll(getSolvedExpXorUpdateList(resVar, condList, opCallExp, actContext));

				break;

			default :
		}

		return result;
	}

	/**
	 * @param string
	 * @param condList
	 * @param opCallExp
	 * @return
	 */
	private static ArrayList getSolvedExpXorUpdateList(String var, ArrayList condList, MOclExpression assOclExp,
			Context actContext) throws ConverterException {
		// Diese Methode gibt ein leeres Array zurueck, wenn eine Variable
		// nicht aufgeloest werden kann, weil die sich auf eine zweite bezieht.
		// Dafuer wird in die ConditionList ein neuer Wert eingefuegt, so dass
		// die zweite Variable mit einer MOperationCallExp aufgeloest werden
		// kann.

		// FIXME Navigationen mit Parser trennen und Variablen suchen??
		
		Constraint actConstraint = actContext.getConstraint();

		ArrayList eqns = new ArrayList();

		String opCallType = assOclExp.getType().getName();

		// Variable in der Liste suchen
		// ber deep-copy der condList laufen! Einfgen gleich in condList!
		ArrayList iterCopy = new ArrayList(condList);

		for (Iterator i = iterCopy.iterator(); i.hasNext();) {
			ConditionInfo cInfo = (ConditionInfo) i.next();

			// Wenn die Variablennamen nicht uebereinstimmen, naechste versuchen
			if (!cInfo.getVariable().equals(var)) {
				continue;
			}

			if (!cInfo.getType().equals(opCallType)) {
				throw new ConverterException("Im Constraint '" + actConstraint.getName() + "' mit dem Context '"
						+ actContext.getName() + "'\nist die Variable '" + var + "'\n" + "als '"
						+ opCallType + "' deklariert. Der ihr\n" + "zugewiesene Ausdruck ist jedoch vom Typ '"
						+ cInfo.getType() + "'!");
			}

			// 1. Fall: Die Variable laesst sich direkt mit einem Wert aufloesen
			if (!cInfo.getAssIsVar()) {
				// Typ erzeugen
				MClassifier type = new MClassifierImpl();
				type.setName(cInfo.getType());

				MLiteralExp literalExp = new MStringLiteralExpImpl();
				literalExp.setName(cInfo.getAssociated());
				literalExp.setType(type);
				eqns.add(createEqn(assOclExp, literalExp, cInfo.getOperation()));
			} else {
				// 2. Fall: Eine OpCallExp wurde schon gesetzt
				// dann daraus die Gleichung bauen
				if (cInfo.getOpCall() != null) {
					eqns.add(createEqn(cInfo.getOpCall(), assOclExp, cInfo.getOperation()));
				} else {
					// 3. Fall: Die ConditionList muss erweitert werden.
					// Das Ergebnis ist diesmal leer, damit die Gleichung nicht
					// doppelt entsteht
					ConditionInfo addCInfo = new ConditionInfo(cInfo.getAssociated(), cInfo.getOperation(), null, cInfo
							.getType(), true);
					addCInfo.setAssOclExp(assOclExp);
					condList.add(addCInfo);
				}
			}
		}

		return eqns;
	}

	private static boolean containSuperPath(ArrayList paths, GraphPath subPath) {
		Iterator dubtest = paths.iterator();
		GraphPath actPath;

		while (dubtest.hasNext()) {
			actPath = (GraphPath) dubtest.next();

			if (actPath.isSuperPathOf(subPath)) {
				return true;
			}
		}

		return false;
	}

	private static ArrayList navableAssociationEndRoles(ClassifierNode clNode) {
		ArrayList list = new ArrayList();

		AssociationEndRoleSource actAssEndRoleSource;
		AssociationEndRoleTarget actAssEndRoleTarget;

		EList assEndRoles = clNode.getSource();

		if (assEndRoles != null) {
			// Alle Sources der ClassifierRole durchsuchen
			// System.out.println("Sources: " + clNode.getSource().size());
			for (Iterator i = assEndRoles.iterator(); i.hasNext();) {
				actAssEndRoleSource = (AssociationEndRoleSource) i.next();
				actAssEndRoleTarget = actAssEndRoleSource.getAssociationRole().getTarget();

				// System.out.println("Pruefe Source: " + actAssEndRoleTarget);
				// Welche TargetAssociationEndRole ist navigierbar?
				String actAssEndRoleName = actAssEndRoleTarget.getName();
				if (!(actAssEndRoleName.equals("") || actAssEndRoleName.toLowerCase().equals("isin"))) {
					// System.out.println("hinzugefuegte Source: " + actAssEndRoleTarget);
					list.add(actAssEndRoleTarget);
				}
			}
		}
		// Alle Targets der ClassifierRole durchsuchen
		// System.out.println("Targets: " + clNode.getTarget().size());

		assEndRoles = clNode.getTarget();

		if (assEndRoles != null) {
			for (Iterator i = assEndRoles.iterator(); i.hasNext();) {
				actAssEndRoleTarget = (AssociationEndRoleTarget) i.next();
				actAssEndRoleSource = actAssEndRoleTarget.getAssociationRole().getSource();

				// System.out.println("Pruefe Target: " + actAssEndRoleSource);
				// Welche SourceAssociationEndRole ist navigierbar?
				String actAssEndRoleName = actAssEndRoleSource.getName();
				if (!(actAssEndRoleName.equals("") || actAssEndRoleName.toLowerCase().equals("isin"))) {
					// System.out.println("hinzugefuegte Target: " + actAssEndRoleSource);
					list.add(actAssEndRoleSource);
				}
			}
		}

		return list;
	}

	private static MOperationCallExp createEqn(MOclExpression arg1, MOclExpression arg2, String op) {
		// Booleantyp erzeugen
		MClassifier booleanType = new MClassifierImpl();
		booleanType.setName("Boolean");

		// Gleichung erzeugen
		MOperationCallExp eqn = new MOperationCallExpImpl();
		eqn.setType(booleanType);

		//Operation op an Gleichung binden
		MOperation eqnOperation = new MOperationImpl();
		eqnOperation.setName(op);
		eqn.setReferredOperation(eqnOperation);

		// Argumente in die Gleichung einfuegen
		eqn.addArguments(arg1);
		eqn.addArguments(arg2);
		arg1.setParentOperation(eqn);
		arg2.setParentOperation(eqn);

		return eqn;
	}

	private static ArrayList createAttribExp(GraphPath navPath, EList attribs, ArrayList condList, Context actContext)
			throws ConverterException {
		// createAttribExp testen
		ArrayList expList = new ArrayList();

		Iterator iattr = attribs.iterator();
		Attribute actAttrib;

		// Alle Attribute untersuchen
		while (iattr.hasNext()) {
			actAttrib = (Attribute) iattr.next();

			// Navigationsbaum ueber die AssociationEndRoles aufbauen
			MOclExpression assEndCallExp = createAssEndCallExpTree(navPath);

			// Classifier des Typs bauen
			MClassifier classifier = new MClassifierImpl();
			classifier.setName(actAttrib.getType());

			// "Linke Seite" zusammenstellen, mit Attributname erweitern
			MAttributeCallExp attributeCallExp = new MAttributeCallExpImpl();
			attributeCallExp.setSource(assEndCallExp);
			assEndCallExp.setAppliedProperty(attributeCallExp);
			attributeCallExp.setType(classifier);

			MAttribute attribute = new MAttributeImpl();
			attribute.setName(actAttrib.getName());
			attributeCallExp.setReferredAttribute(attribute);

			VariableDeclaration actVar = actAttrib.getVariableDeclaration();

			// ist der "Wert" des Attributs eine Variable?
			if (actVar != null) {
				// Variable aufloesen, Gleichungen erzeugen
				expList.addAll(getSolvedExpXorUpdateList(actVar.getVarName(), condList, attributeCallExp, actContext));
			} else {
				// sonst Wert direkt einfuegen
				// "Rechte Seite" vorbereiten
				MLiteralExp literalExp = new MStringLiteralExpImpl();
				literalExp.setName(actAttrib.getValue());
				literalExp.setType(classifier);

				// Gleichung aufstellen
				MOperationCallExp exp = createEqn(attributeCallExp, literalExp, "=");

				expList.add(exp);
			}
		}

		return expList;
	}

	private static MOperationCallExp createSelfCycleExp(GraphPath cyclePath) {
		// Classifier des Typs bauen (immer String zur Vereinfachung)
		MClassifier classifier = new MClassifierImpl();
		classifier.setName("String");

		// "Linke Seite" (Navigationsexpression) erzeugen
		MOclExpression assEndCallExp = createAssEndCallExpTree(cyclePath);
		assEndCallExp.setType(classifier);

		// "Rechte Seite" aufbauen
		MVariableExp varExp = new MVariableExpImpl();
		MVariableDeclaration varDecl = new MVariableDeclarationImpl();
		varExp.setType(classifier);
		varExp.setReferredVariable(varDecl);
		varDecl.setVarname(cyclePath.getSource().getCallVar().getVarName());

		// Gleichung aufstellen
		return createEqn(assEndCallExp, varExp, "=");
	}

	private static ArrayList createEquivExp(GraphPath path, ArrayList equivPaths, ArrayList navEqns) {
		ArrayList expList = new ArrayList();

		// Typ-Classifier erzeugen
		MClassifier classifier = new MClassifierImpl();
		classifier.setName(path.getActual().getType());

		// Linke Seite Navigationsbaum ueber die AssociationEndRoles aufbauen
		MOclExpression arg1 = createAssEndCallExpTree(path);
		arg1.setType(classifier);

		/*
		 * try { System.out.println(arg1.convert2OclString()); } catch (NoSuchAttributeException e) {
		 * e.printStackTrace(); }
		 */
		GraphPath actPath;

		// Das Ende des Weges kann eine Collection sein!
		ClassifierNode actual = path.getActual();
		boolean duplicate;
		try {
			for (Iterator iequiv = equivPaths.iterator(); iequiv.hasNext();) {
				actPath = (GraphPath) iequiv.next();

				// Rechte Seite (Teilnavigation) erzeugen
				MOclExpression arg2 = createAssEndCallExpTree(actPath, actual);
				arg2.setType(classifier);

				MOperationCallExp eqn = createEqn(arg1, arg2, "=");

				// Doppelte Wege filtern
				String str1 = eqn.convert2OclString();
				duplicate = false;

				for (Iterator i = navEqns.iterator(); i.hasNext();) {

					String str2 = ((MOclExpression) i.next()).convert2OclString();

					//System.out.println(" Teste: " + str1 + " vs. " + str2);
					if (str1.equals(str2)) {
						//System.out.println("Doppelter Weg gefunden!");
						duplicate = true;
						break;
					}

					for (Iterator j = expList.iterator(); j.hasNext();) {
						String str3 = ((MOclExpression) j.next()).convert2OclString();

						//System.out.println(" Teste: " + str1 + " vs. " +
						// str3);
						if (str1.equals(str3)) {
							//System.out.println("Doppelter Weg gefunden!");
							duplicate = true;
							break;
						}
					}

				}

				//System.out.println(" Aeq. Weg: " + str1);
				if (!duplicate) {
					expList.add(eqn);
				}
			}
		} catch (NoSuchAttributeException e1) {
			// Das sollte nicht auftreten, wenn die OCL-Metamodellinstanzen korrekt zusammengebaut werden
			e1.printStackTrace();
		}

		return expList;
	}

	private static MOperationCallExp createConj() {
		// -- Booleantyp erzeugen
		MClassifier booleanType = new MClassifierImpl();
		booleanType.setName("Boolean");

		// Konjunktion erzeugen
		MOperationCallExp conj = new MOperationCallExpImpl();
		conj.setType(booleanType);

		// -- Operation and an Konjunktion binden
		MOperation conjOperation = new MOperationImpl();
		conjOperation.setName("and");
		conj.setReferredOperation(conjOperation);

		return conj;
	}

	private static MOclExpression createAssEndCallExpTree(GraphPath path) {
		// Baum von vorne ab der self-Variablen bis zum Ende der AssEndRole-Liste aufbauen
		ClassifierNode pathSource = path.getSource();
		MOclExpression actSource = null;
		
		MClassifier classifier = new MClassifierImpl();
		classifier.setName(pathSource.getType());
		
		MVariableExp varExp = new MVariableExpImpl();
		MVariableDeclaration varDecl = new MVariableDeclarationImpl();
		varExp.setReferredVariable(varDecl);
		varExp.setType(classifier);
		
		if (pathSource instanceof ClassifierRole) {
			// Bei einer ClassifierRole ist die Basis eine VariableExpression
			varDecl.setVarname(path.getSource().getCallVar().getVarName());
			
			// Hilfszeiger
			actSource = varExp;
		
			for (int i = 0; i < path.length(); i++) {
				// Navigationsschritt erzeugen und vorige anhaengen
				MAssociationEndCallExp assEndCallExp = new MAssociationEndCallExpImpl();
				MAssociationEnd assEnd = new MAssociationEndImpl();
				assEnd.setName(path.getLink(i).getName());

				//System.out.println("AssEndCallBaum aktuell:" + assEnd);
				assEndCallExp.setReferredAssociationEnd(assEnd);
				assEndCallExp.setSource(actSource);
				actSource = assEndCallExp;
			}
		} else {
			// Bei einer SetClassifier ist die Basis diese mit allInstances() und es gibt keine Navigation!
			varDecl.setVarname(path.getSource().getType());
			MOperationCallExp allInstOpCallExp = new MOperationCallExpImpl();
			MSetType setType = new MSetTypeImpl();
			
			setType.setElementType(classifier);
			
			allInstOpCallExp.setType(setType);
			allInstOpCallExp.setSource(varExp);
			MOperation allInstOp = new MOperationImpl();
			allInstOp.setName("allInstances");
			allInstOpCallExp.setReferredOperation(allInstOp);			
			actSource = allInstOpCallExp;
		}

		return actSource;
	}

	private static MOclExpression createAssEndCallExpTree(GraphPath path, ClassifierNode stopClassifier) {
		// Baum von vorne ab der self-Variablen bis zum bergebenen ClassifierNode aufbauen
		MVariableExp varExp = new MVariableExpImpl();
		MVariableDeclaration varDecl = new MVariableDeclarationImpl();

		varExp.setReferredVariable(varDecl);
		varDecl.setVarname(path.getSource().getCallVar().getVarName());

		// Hilfszeiger
		MOclExpression actSource = varExp;
		int i = 0;

		do {
			// Navigationsschritt erzeugen und vorige anhaengen
			MAssociationEndCallExp assEndCallExp = new MAssociationEndCallExpImpl();
			MAssociationEnd assEnd = new MAssociationEndImpl();
			assEnd.setName(path.getLink(i).getName());

			//System.out.println("AssEndCallBaum aktuell:" + assEnd);
			assEndCallExp.setReferredAssociationEnd(assEnd);
			assEndCallExp.setSource(actSource);
			actSource = assEndCallExp;
		} while (path.getLink(i++).getType() != stopClassifier);

		return actSource;
	}
	/*
	 * private static void show(ArrayList paths) { System.out.print("Liste: ["); for (Iterator i = paths.iterator();
	 * i.hasNext();) { ((GraphPath) i.next()).show(); System.out.print(" "); }
	 * 
	 * System.out.println("]"); }
	 */
}