package de.tuberlin.tfs.ocl.metamodel.expressions;

import java.util.*;
import de.tuberlin.tfs.ocl.metamodel.types.*;
import de.tuberlin.tfs.ocl.metamodel.converter.*;
import ru.novosoft.uml.foundation.core.*;
// import ru.novosoft.uml.foundation.data_types.*;
// import ru.novosoft.uml.behavior.common_behavior.*;


public class MOclConstraintImpl extends MModelElementImpl implements MOclConstraint {

	protected MOclExpression body;
	protected MOclConstraintKind kind;
        protected MModelElement constrainedElement;


	/** Getter for property body.
	 * @return Value of property body.
	 */
	public MOclExpression getBody() {
		return body;
	}

	/** Setter for property body.
	 * @param body value of property body.
	 */
	public void  setBody(MOclExpression body) {
		this.body = body;
	}

	/** Getter for property kind.
	 * @return Value of property kind.
	 */
	public MOclConstraintKind getKind() {
		return kind;
	}

	/** Setter for property kind.
	 * @param kind value of property kind.
	 */
	public void  setKind(MOclConstraintKind kind) {
		this.kind = kind;
	}

        /** Getter for property constraindeElement.
         * @return Value of property constraindeElement.
         */
        public MModelElement getConstrainedElement() {
            return constrainedElement;
        }

        /** Setter for property constraindeElement.
         * @param kind value of property constraindeElement.
         */
        public void setConstrainedElement(MModelElement constrainedElement) {
            this.constrainedElement = constrainedElement;
        }

	/** String converter for OCL Constraints
	 * @return OCL constraint string
	 */
	public String convert2OclString () throws NoSuchAttributeException {
		// say hello
		OclConverter.message("-= OCL Converter V" + OclConverter.version + "=-");
		OclConverter.message("<MOclConstraint>");
		StringBuffer strB = new StringBuffer();
		strB.append("context ");
		OclConverter.checkAttribute(constrainedElement, "constrained element", this);

		if (constrainedElement instanceof MOperationCallExp) {

			OclConverter.message("instance of MOperationCallExp");

			MOperationCallExp constrElem = (MOperationCallExp) constrainedElement;

			// there must be a referred operation (see above)
			OclConverter.checkAttribute(constrElem.getReferredOperation(),
				"referred operation", this);
			OclConverter.checkAttribute(constrElem.getReferredOperation().getName(),
				 "name of referred operation", this);
			OclConverter.checkAttribute(constrElem.getType(), "type of operation", this);
			OclConverter.checkAttribute(constrElem.getType().getName(), "name of type of operation", this);



			OclConverter.message("referredOperation: " + constrElem.getReferredOperation().getName());
			strB.append(constrElem.getReferredOperation().getName() + "(");

			// look for source
			if (constrElem.getSource() != null) {
				OclConverter.warning("source found, but not usable in this context -> skipping.");
			}

			// arguments at all?
			if (constrElem.getArguments() == null ||
				(constrElem.getArguments() != null && constrElem.getArguments().size() == 0) ) {

				if (constrElem.getArguments() == null) OclConverter.warning("missing arguments; proceeding...");

				OclConverter.message("0 arguments");
			}
			else {

				// argument != null && argument.size() > 0 !
				Iterator i = constrElem.getArguments().iterator();

				// get first argument (often used)
				MOclExpression exprArg = (MOclExpressionImpl) i.next();

				// exprArg must have a type!
				OclConverter.checkAttribute(exprArg.getType(), "type of first argument", this);

				// operator, bracket, 2nd expression
				strB.append (exprArg.convert2OclString() + ":" + exprArg.getType().getName() );

				// further expressions, if available
				while (i.hasNext() ) {
					exprArg = (MOclExpressionImpl) i.next();

					OclConverter.checkAttribute(exprArg.getType(), "type of an argument", this);

					strB.append ("," + exprArg.convert2OclString() + ":" + exprArg.getType().getName() );
				}

			} // else

			// closing bracket and type
			strB.append ("):" + constrElem.getType().getName() );

		}
		else {
			OclConverter.checkAttribute(constrainedElement.getName(),
		   		"name of constrained element", this);
			strB.append(constrainedElement.getName() );
		}



		OclConverter.checkAttribute(kind, "kind", this);
		OclConverter.checkAttribute(kind.getName(), "name of kind", this);

		strB.append(" " + kind.getName() );
		if (getName() != null) strB.append (" " + getName());
		strB.append(": ");

		OclConverter.checkAttribute(body, "body", this);

		strB.append(body.convert2OclString() );

		OclConverter.message("</MOclConstraint>");

		return strB.toString();
	}

}
