/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.action;

import java.util.List;


import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IWorkbenchPart;
import VOCLModel.ClassifierRole;
import vocleditor.action.request.ClassifierRolePropertiesRequest;
import vocleditor.commands.adds.ClassifierPropsCommand;
import vocleditor.edit.NavExpContainerEditPart;
import vocleditor.edit.ClassifierRoleEditPart;


/**
 * A generic action implementation for dealing with simpleclassifierrole.
 */
public class PropertiesForClassifierRoleAction extends SelectionAction
{
    // The command type as string value
    private final String commandType;

    // The name shown in the Context-Menu
    public static final String CLASSIFIER_PROPERTY = "Properties";

    /**
     * Creates a new AttributeAction instance.
     * @param part 
     * @param commandType
     */
    public PropertiesForClassifierRoleAction(IWorkbenchPart part, String commandType)
    {
        super(part);
        this.commandType = commandType;
        setId(commandType);
        setText(commandType);
		setImageDescriptor(ImageDescriptor.createFromFile(
							PropertiesForClassifierRoleAction.class,
												"gif/simplecl.gif"));
    }

    /**
     * Calculates if this action is enabled, i.e. if the user should
     * be able to execute this action. This is the case, if
     * a command for this action can be instanciated (createAddAttributeCommand)
     * and the command can be executed (cmd.canExecute()). 
     */
    protected boolean calculateEnabled()
    {
        Command cmd = createAddPropertiesCommand(getSelectedObjects());
        if (cmd == null)
            return false; 
        return cmd.canExecute();
    }

    /**
     * Creates the coresponding command. This only is done, if one
     * ClassifierRoleEditPart is selected on the work-bench.
     * @param list - the list of selected edit-parts
     * @return - an PropertyCommand
     */
    protected Command createAddPropertiesCommand(List selectedEditParts)
    {
    	// Feature is only available for 
        if (selectedEditParts.size() != 1)
        	return null;
        
        // get the EditPart
		ClassifierRoleEditPart ep = null;
        
        // check the EditPart for
        // only allowed for ClassifierRoleEditPart 
        if (!(selectedEditParts.get(0) instanceof ClassifierRoleEditPart))
        	return null;
        
        ep = (ClassifierRoleEditPart)selectedEditParts.get(0);
        
        // Create the Request	
        ClassifierRolePropertiesRequest attributeReq = new ClassifierRolePropertiesRequest(getCommandType());
		// Create and return the command
		ClassifierPropsCommand cmd = null;
		if (ep.understandsRequest(attributeReq)) {
			Command c = ep.getCommand(attributeReq);
			if (c instanceof ClassifierPropsCommand) {
				cmd = (ClassifierPropsCommand)c;
				cmd.setSource((ClassifierRole)ep.getClassifierNode());
				cmd.setContainer(((NavExpContainerEditPart)ep.getParent()).getNavExpContainer());
			}
		}

        return cmd;
    }

    /**
     * Returns the command type.
     * @return the command type
     */
    public String getCommandType()
    {
        return commandType;
    }

    /**
     * Simply calls this.execute()
     */
    public void run()
    {
        execute(createAddPropertiesCommand(getSelectedObjects()));
    }

    /**
     * Initializes this action.
     */
    protected void init()
    {
        super.init();
        setText(getCommandType());
        setToolTipText(getCommandType());
        setId(getCommandType());
        setEnabled(false);
    }
    
}
