/*
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.action;

import java.util.List;

// import org.eclipse.draw2d.Figure;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IWorkbenchPart;

import VOCLModel.SetClassifierRole;
import vocleditor.action.request.SetClassifierRolePropertiesRequest;
import vocleditor.commands.adds.SetClassifierPropsCommand;
import vocleditor.edit.SetClassifierRoleEditPart;
import vocleditor.edit.NavExpContainerEditPart;
import vocleditor.figure.SetClassifierRoleFigure;

/**
 * A generic action implementation for dealing with simpleclassifierrole.
 */
public class PropertiesForSetClassifierRoleAction extends SelectionAction
{
    // The command type as string value
    private final String commandType;

    // The name shown in the Context-Menu
    public static final String CLASSIFIER_PROPERTY = "Set Properties";

    /**
     * Creates a new PropertiesForCollClassifierRoelAction instance.
     * @param part - the Editor main-part
     * @param commandType - the String for this command
     */
    public PropertiesForSetClassifierRoleAction(IWorkbenchPart part, String commandType)
    {
        super(part);
        this.commandType = commandType;
        setId(commandType);
        setText(commandType);
		setImageDescriptor(ImageDescriptor.createFromFile(
									PropertiesForSetClassifierRoleAction.class,
														"gif/simplecl.gif"));
    }

    /**
     * Calculates if this action is enabled, i.e. if the user should
     * be able to execute this action. This is the case, if
     * a command for this action can be instanciated (createAddAttributeCommand)
     * and the command can be executed (cmd.canExecute()). 
     */
    protected boolean calculateEnabled()
    {
        Command cmd = createAddPropertiesCommand(getSelectedObjects());
        if (cmd == null)
            return false; 
        return cmd.canExecute();
    }

    /**
     * Creates the coresponding command. This only is done, if one
     * ClassifierRoleEditPart is selected on the work-bench.
     * @param list - the list of selected edit-parts
     * @return - an PropertyCommand
     */
    protected Command createAddPropertiesCommand(List selectedEditParts)
    {
    	// Feature is only available for 
        if (selectedEditParts.size() != 1)
        	return null;
        
        // get the EditPart
		SetClassifierRoleEditPart ccerp = null;
        
        // check the EditPart for
        // only allowed for ClassifierRoleEditPart 
        if (!(selectedEditParts.get(0) instanceof SetClassifierRoleEditPart))
        	return null;
        
		ccerp = (SetClassifierRoleEditPart)selectedEditParts.get(0);
        
        // Create the Request	
        SetClassifierRolePropertiesRequest attributeReq = new SetClassifierRolePropertiesRequest(getCommandType());
		// Create and return the command
		
		SetClassifierPropsCommand cmd = null;
		if (ccerp.understandsRequest(attributeReq)) {
			Command c = ccerp.getCommand(attributeReq);
			if (c instanceof SetClassifierPropsCommand) {
				cmd = (SetClassifierPropsCommand)c;
				cmd.setSource((SetClassifierRole)ccerp.getClassifierNode());
				cmd.setContainer(((NavExpContainerEditPart)ccerp.getParent()).getNavExpContainer());
			}
		}
		
		// add the figure to the command
		SetClassifierRoleFigure crf = (SetClassifierRoleFigure)ccerp.getFigure();
		cmd.setClassFigure(crf);
		
        return cmd;
    }

    /**
     * Returns the command type.
     * @return the command type
     */
    public String getCommandType()
    {
        return commandType;
    }

    /**
     * Simply calls this.execute()
     */
    public void run()
    {
        execute(createAddPropertiesCommand(getSelectedObjects()));
    }

    /**
     * Initializes this action.
     */
    protected void init()
    {
        super.init();
        setText(getCommandType());
        setToolTipText(getCommandType());
        setId(getCommandType());
        setEnabled(false);
    }
    
}
