/*
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.action;

import java.util.List;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IWorkbenchPart;

import vocleditor.action.request.VOCLContainerPropertiesRequest;
import vocleditor.commands.adds.AddConditionCommand;
import vocleditor.edit.ConditionEditPart;
import vocleditor.edit.NavExpContainerEditPart;
import VOCLModel.Constraint;
import VOCLModel.NavExpContainer;
import VOCLModel.VOCLExpression;



/**
 * A generic action implementation for dealing with simpleclassifierrole.
 */
public class PropertiesForVOCLContainerAction extends SelectionAction
{
    // The command type as string value
    private final String commandType;

    // The name shown in the Context-Menu
    public static final String CLASSIFIER_PROPERTY = "conditions";

    /**
     * Creates a new add condition action instance.
     * @param part 
     * @param commandType
     */
    public PropertiesForVOCLContainerAction(IWorkbenchPart part, String commandType)
    {
        super(part);
        this.commandType = commandType;
        setId(commandType);
        setText(commandType);
		setImageDescriptor(ImageDescriptor.createFromFile(
							PropertiesForVOCLContainerAction.class,
												"gif/condition.gif"));
    }

    /**
     * Calculates if this action is enabled, i.e. if the user should
     * be able to execute this action. This is the case, if
     * a command for this action can be instanciated (AddConditiionCommand)
     * and the command can be executed (cmd.canExecute()). 
     */
    protected boolean calculateEnabled()
    {
        Command cmd = createAddConditionCommand(getSelectedObjects());
        if (cmd == null)
            return false; 
        return cmd.canExecute();
    }

    /**
     * Creates the coresponding command. This only is done, if one
     * ClassifierRoleEditPart is selected on the work-bench.
     * @param list - the list of selected edit-parts
     * @return - an PropertyCommand
     */
    protected Command createAddConditionCommand(List selectedEditParts)
    {
    	// Feature is only available for 
        if (selectedEditParts.size() != 1)return null;
        
        // get the EditPart
		NavExpContainerEditPart ep = null;
        ConditionEditPart  conep=null;
        
        // check the EditPart for
        // only allowed for ClassifierRoleEditPart 
        if ((selectedEditParts.get(0) instanceof NavExpContainerEditPart))
			ep = (NavExpContainerEditPart)selectedEditParts.get(0);
		else if(selectedEditParts.get(0) instanceof ConditionEditPart)	
			conep = (ConditionEditPart)selectedEditParts.get(0);
		else return null;
        	  
        // Create the Request	
        VOCLContainerPropertiesRequest req = new VOCLContainerPropertiesRequest(getCommandType());
		// Create and return the command
		AddConditionCommand cmd = null;
		if(ep!=null){
			if (ep.understandsRequest(req)) {
				Command c = ep.getCommand(req);
				if (c instanceof AddConditionCommand) {
					cmd = (AddConditionCommand)c;
					VOCLExpression exp=ep.getNavExpContainer();
					cmd.setContainer((NavExpContainer) exp);
					Constraint constraint=null;
					while(constraint==null){
						constraint=exp.getConstraint();
						if(constraint==null)exp=(VOCLExpression) exp.eContainer();
					}
					cmd.setConstraint(constraint);
				}
			}
		}else{
			if (conep.understandsRequest(req)) {
				Command c = conep.getCommand(req);
				if (c instanceof AddConditionCommand) {
					cmd = (AddConditionCommand)c;
					VOCLExpression exp=conep.getCondition().getNavExpContainer();
					cmd.setContainer((NavExpContainer) exp);
					Constraint constraint=null;
					while(constraint==null){
						constraint=exp.getConstraint();
						if(constraint==null)exp=(VOCLExpression) exp.eContainer();
					}
					cmd.setConstraint(constraint);					
				}
			}				
		}
        return cmd;
    }

    /**
     * Returns the command type.
     * @return the command type
     */
    public String getCommandType(){
        return commandType;
    }

    /**
     * Simply calls this.execute()
     */
    public void run(){
        execute(createAddConditionCommand(getSelectedObjects()));
    }

    /**
     * Initializes this action.
     */
    protected void init()
    {
        super.init();
        setText(getCommandType());
        setToolTipText(getCommandType());
        setId(getCommandType());
        setEnabled(false);
    }
    
}
