/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.commands.adds;



import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.gef.commands.Command;

import vocleditor.model.ConditionElement;
import vocleditor.model.VOCLModelManager;
import VOCLModel.Condition;
import VOCLModel.Constraint;
import VOCLModel.NavExpContainer;
import VOCLModel.VOCLModel4Factory;
import VOCLModel.VariableDeclaration;



public class AddConditionCommand extends Command {
	
	// Names for Label and Description of this Command
	private static final String ConnectionCommand_Label 		= "conditions";
	private static final String ConnectionCommand_Description 	= "conditions in container";

	private boolean RightSideVar=false;
	private VariableDeclaration left;
	private VariableDeclaration right;
	private List conditionsList=new LinkedList();
	private List oldConditionsList=new LinkedList();
	private Constraint constraint;
	protected NavExpContainer container;	
	
	
	/**
	 * Constructor, simply calls the superclass
	 * with the ConnectionLabel as parameter.
	 */
	public AddConditionCommand() {
		super(ConnectionCommand_Label);
	}
	

	/**
	 * This method returns true, if a source (ClassifierRole, see class-members)
	 * is specified. 
	 */
	public boolean canExecute() {
		if (container == null)
			return false;
		else
			return true;
	}
	
	/**
	 * This method handles the execution of adding a condition.
	 */
	public void execute() {
		createList(oldConditionsList);
		insertList(conditionsList);
	}


	/**
	 * Inserting the list of elements into this
	 * condition.
	 * @param list - the list of elements
	 */
	private void insertList(List list){
		VOCLModel4Factory mfc = VOCLModelManager.getFactory();	
		Iterator i=container.getCondition().iterator();
		while(i.hasNext()){
			Condition cond=(Condition) i.next();
			constraint.getVariableDeclaration().remove(cond.getLeft());
			if(cond.getValue().trim().length()==0)
				constraint.getVariableDeclaration().remove(cond.getRight());				
		}	
		container.getCondition().clear();
		Iterator j=list.iterator();
		while(j.hasNext()){
			Condition condition=mfc.createCondition();
			ConditionElement con=(ConditionElement)j.next();
			condition.setOp(con.getOp());
			VariableDeclaration left=mfc.createVariableDeclaration();
			left.setVarName(con.getLeftVarName());
			left.setType(con.getLeftVarType());
			constraint.getVariableDeclaration().add(left);
			condition.setLeft(left);
			if(con.isRightVar()){
				VariableDeclaration right=mfc.createVariableDeclaration();
				right.setVarName(con.getRightVarName());
				right.setType(con.getRightVarType());
				constraint.getVariableDeclaration().add(right);
				condition.setRight(right);
				condition.setValue("");
			}else condition.setValue(con.getValue());
			container.getCondition().add(condition);			
			
		}						
	}


	/**
	 * Retrieves the description.
	 */
	public String getLabel() {
		return ConnectionCommand_Label; // Description; 
	}



	/**
	 * Redo this action means 
	 * simply execute it (again).
	 */
	public void redo() {
		insertList(conditionsList);
	}


	/**
	 * inserting elements of old list
	 */
	public void undo() {
		insertList(oldConditionsList);
	}
	
	/**
	 * Returns the container-class of this model.
	 * @return - a NavExpContainer
	 */
	public NavExpContainer getContainer() {
		return container;
	}

	/**
	 * Sets the container-class for this model.
	 * @param container
	 */
	public void setContainer(NavExpContainer container) {
		this.container = container;
	}
	
	/**
	 * Getter for the constraint
	 * @return - the responsible constraint 
	 */
	public Constraint getConstraint() {
		return constraint;
	}

	/**
	 * Setter for the constraint
	 * @param constraint - the responsible constraint
	 */
	public void setConstraint(Constraint constraint) {
		this.constraint = constraint;
	}


	/**
	 * Ceates a list with conditions
	 * @param list - the list, which shall be created with conditions
	 */
	private void createList(List list){
		list.clear();
		Iterator i=getContainer().getCondition().iterator();
		while(i.hasNext()){
			Condition condition=(Condition) i.next();;
			ConditionElement con=new ConditionElement();
			VariableDeclaration left =condition.getLeft();
			con.setLeftVarName(left.getVarName());
			con.setLeftVarType(left.getType());
			con.setOp(condition.getOp());
			con.setValue(condition.getValue());
			if(condition.getValue().trim().length()==0){
				VariableDeclaration right =condition.getRight();
				con.setRightVarName(right.getVarName());
				con.setRightVarType(right.getType());
				con.setRightVar(true);
			}
			list.add(con);
		}						
	}
	
	/**
	 * Getter for the conditionList
	 * @return - the list containing all conditions
	 */
	public List getConditionsList() {
		createList(conditionsList);
		return conditionsList;
	}

	/**
	 * Setter for the conditionList
	 * @param list - the new list of conditions
	 */
	public void setConditionsList(List list) {
		conditionsList.clear();
		conditionsList.addAll(list);
	}

}