/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.commands.create;


import org.eclipse.draw2d.geometry.Rectangle;

import vocleditor.model.VOCLModelManager;
import VOCLModel.ClassifierNode;
import VOCLModel.Constraint;
import VOCLModel.NavExpContainer;
import VOCLModel.VOCLModel4Factory;
import VOCLModel.VariableDeclaration;


public class CreateClassifierNodeCommand
	extends org.eclipse.gef.commands.Command
{
	private static final String	CreateCommand_LabelSimple = "CreateClassifierRoleCommand";
	private static final String	CreateCommand_LabelColl = "CreateCollClassifierRoleCommand";
	
	private ClassifierNode classifiernode;
	private Rectangle loc;
	private NavExpContainer parent;
	private VariableDeclaration vdcl;
	private Constraint constraint;
	
	private String nameOfClassifierNode;
	private String typeOfClassifierNode;
	
	/**
	 * Constructor of this command, 
	 */
	public CreateClassifierNodeCommand() {
		super();
	}

	/**
	 * The execution of this command creates a new ClassifierNode
	 * (Simple or Collection)
	 */
	public void execute() {
		createDiagramm();
		if (loc != null) {
			classifiernode.setX(loc.x);
			classifiernode.setY(loc.y);
		}
	}
	
	/**
	 * Implementation of the creation of a 
	 * ClassifierNode
	 */
	private void createDiagramm(){
		classifiernode.setType(getTypeOfClassifierRole());
		parent.getClassifierNode().add(classifiernode);
		if(getNameOfClassifierRole().trim().length()>0){
			VOCLModel4Factory mfc = VOCLModelManager.getFactory();
			vdcl = mfc.createVariableDeclaration();
			vdcl.setVarName(getNameOfClassifierRole());
			vdcl.setType(getTypeOfClassifierRole());
			classifiernode.setCallVar(vdcl);	
			constraint.getVariableDeclaration().add(vdcl);	
		}
	}
	
	/**
	 * Getterfor the container, where this node is nested in
	 * @return - the NavExpContainer where this node is nested in
	 */
	public NavExpContainer getParent() {
		return parent;
	}
	
	
	/**
	 * Redos a previously made undo of this command
	 */
	public void redo() {
		if(vdcl!=null){
			classifiernode.setCallVar(vdcl);	
			constraint.getVariableDeclaration().add(vdcl);
		}
		parent.getClassifierNode().add(classifiernode); 
	}
	
	/**
	 * Setter for a classifierNode
	 * @param clr - the new classifierNode
	 */
	public void setClassiefierNode(ClassifierNode clr ) {
		classifiernode = clr;
	}
	
	/**
	 * Sets a new location of a classifierNode
	 * @param loc - a rectangle holding the new location
	 */
	public void setLocation (Rectangle loc) {
		this.loc = loc;
	}
	
	/**
	 * Sets the container, where the classifierNode 
	 * should be placed inside
	 * @param newParent - the new parent
	 */
	public void setParent(NavExpContainer newParent) {
		parent = newParent;
	}
	
	/**
	 * Invalidates the actions done by this command
	 */
	public void undo() {
		if(vdcl!=null){
			classifiernode.setCallVar(null);	
			constraint.getVariableDeclaration().remove(vdcl);
		}	
		parent.getClassifierNode().remove(classifiernode);
	}
	

	/**
	 * Getter for the name of the created classifierRole
	 * @return - the string-value of the classifierRoles name
	 */
	public String getNameOfClassifierRole() {
		return nameOfClassifierNode;
	}

	/**
	 * Retrieves the type of the classifierRole
	 * @return - the string-value of the classfierRoles type.
	 */
	public String getTypeOfClassifierRole() {
		return typeOfClassifierNode;
	}

	/**
	 * Setter for the name of the classifierNode
	 * @param string - the new name
	 */
	public void setNameOfClassifierNode(String string) {
		nameOfClassifierNode = string;
	}

	/**
	 * Setter for the type of the classifierNode
	 * @param string - the new type as string-value
	 */
	public void setTypeOfClassifierNode(String string) {
		typeOfClassifierNode = string;
	}
	
	/**
	 * Getter for the constraint of this classifierNode 
	 * @return - the constraint
	 */
	public Constraint getConstraint() {
		return constraint;
	}

	/**
	 * Setter of the constraint of this classifierNode
	 * @param constraint - the new constraint
	 */
	public void setConstraint(Constraint constraint) {
		this.constraint = constraint;
	}

}
