/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.commands.create;

import java.util.Hashtable;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;

import vocleditor.editors.VOCLEditorConstants;
import vocleditor.model.VOCLModelManager;
import VOCLModel.AssociationEndRoleSource;
import VOCLModel.AssociationEndRoleTarget;
import VOCLModel.AssociationIsIn;
import VOCLModel.ClassifierRole;
import VOCLModel.Constraint;
import VOCLModel.ExpressionType;
import VOCLModel.NavExpContainer;
import VOCLModel.PredicateContainer;
import VOCLModel.PredikatType;
import VOCLModel.SetClassifierRole;
import VOCLModel.VOCLExpression;
import VOCLModel.VOCLModel4Factory;
import VOCLModel.VariableDeclaration;

public class CreatePredicatContainerCommand
								extends org.eclipse.gef.commands.Command{
		
	private static final String	CreateCommand_Label = "CreateCSelectionContainerCommand";

	//NavExpContainer where the selcontainer will be placed
	private NavExpContainer navcontainer;
	//NavExpContainer that  will be insert in the selcontainer
	private NavExpContainer navexp;
	//constriant that extend navcontainer
	private Constraint constraint;
	// location for selcontainer
	private Rectangle loc;
	//dimension for the selcontainer
	private Dimension dim;
	
	//SelectionContainer
	private PredicateContainer selcontainer;
	//predicat of SelectionContainer as CollClassifierRole
	private SetClassifierRole prericatClassifier;
	//if exists,variable of prericatClassifier
	private VariableDeclaration predVar;
	//iterator as ClassifierRole
	private ClassifierRole itClassifier;
	//variable of itClassifier
	private VariableDeclaration iterVar;
	//Assocition isIn between prericatClassifier and itlassifier
	private AssociationIsIn isInassociation;
	//Structure for all parameters of selctioncontainer
	private Hashtable values=new Hashtable();

	/**
	 * The constructor of this command.
	 */
	public CreatePredicatContainerCommand() {
		super();
	}

	/**
	 * Execution of this class creates a new constraint and 
	 * adds it to the container
	 */
	public void execute(){
		if (loc != null){
			selcontainer.setX(loc.x);
			selcontainer.setY(loc.y+40);
			selcontainer.setWidth(200);
			selcontainer.setHeight(100);
			navcontainer.setPredicateContainer(selcontainer);
			VOCLModel4Factory mfc = VOCLModelManager.getFactory();
			navexp=mfc.createNavExpContainer();
			navexp.setExptype(ExpressionType.EXP_LITERAL);
			selcontainer.setVOCLExpression(navexp);
			int sk=((Integer)values.get(VOCLEditorConstants.PREDICAT_TYPE)).intValue();
			selcontainer.setPredType(PredikatType.get(sk));
			
			String name=(String) values.get(VOCLEditorConstants.PREDICAT_SET_NAME);
			String type=(String) values.get(VOCLEditorConstants.PREDICAT_SET_TYPE);
			
			prericatClassifier=mfc.createSetClassifierRole();
			prericatClassifier.setType(type);
			prericatClassifier.setX(loc.x);
			prericatClassifier.setY(loc.y);
			selcontainer.setPredicat(prericatClassifier);
			predVar = mfc.createVariableDeclaration();
			predVar.setVarName(name);
			predVar.setType(type);
			prericatClassifier.setCallVar(predVar);	
			constraint.getVariableDeclaration().add(predVar);	
			navcontainer.getClassifierNode().add(prericatClassifier);
			insertIterators(type);
		}
	}
	
	private void insertIterators(String type){
		VOCLModel4Factory mfc = VOCLModelManager.getFactory();
		String name=(String) values.get(VOCLEditorConstants.PREDICAT_ITERATORS);
		iterVar=mfc.createVariableDeclaration();
		iterVar.setVarName(name);
		iterVar.setType(type);
		selcontainer.setIterator(iterVar);		
		constraint.getVariableDeclaration().add(iterVar);	
		itClassifier=mfc.createClassifierRole();
		itClassifier.setX(5);
		itClassifier.setY(5);
		itClassifier.setType(type);
		itClassifier.setCallVar(iterVar);
		navexp.getClassifierNode().add(itClassifier);

		
		AssociationEndRoleSource assErs=mfc.createAssociationEndRoleSource();
		AssociationEndRoleTarget assErt=mfc.createAssociationEndRoleTarget();
		isInassociation=mfc.createAssociationIsIn();
		navcontainer.getAssociationRole().add(isInassociation);
			
		assErs.setName("");
		isInassociation.setSource(assErs);
		assErs.setAssociationRole(isInassociation);
		assErs.setNumberOfAnchor(3);
			
		assErt.setName("isIn");
		isInassociation.setTarget(assErt);
		assErt.setAssociationRole(isInassociation);
		assErt.setNumberOfAnchor(3);
		
		
		prericatClassifier.getSource().add(assErs);
		itClassifier.getTarget().add(assErt);
		assErs.setType(prericatClassifier);	
		assErt.setType(itClassifier);
	}

	/**
	 * Undos the action
	 */
	public void undo() {	
		navcontainer.setPredicateContainer(null);
		navcontainer.getAssociationRole().remove(isInassociation);
		navcontainer.getClassifierNode().remove(prericatClassifier);
		
		navexp.getClassifierNode().remove(itClassifier);
		
		constraint.getVariableDeclaration().remove(predVar);
		constraint.getVariableDeclaration().remove(iterVar);	

	}	
	
	/**
	 * Redos an undo previously made.
	 */
	public void redo() {
		execute(); 
	}
	
	/**
	 * Setter for the constraint
	 * @param constraint
	 */
	public void setSelectionContainer(PredicateContainer selcontainer ) {
		this.selcontainer = selcontainer;
	}
	
	/**
	 * Setter for the location
	 * @param loc - a point holding the new location
	 */
	public void setLocation (Rectangle loc) {
		this.loc = loc;
	}
	
	private void initValues(Hashtable table){
		table.put(VOCLEditorConstants.PREDICAT_ITERATORS,new String(""));
		table.put(VOCLEditorConstants.PREDICAT_SET_NAME,new String());
		table.put(VOCLEditorConstants.PREDICAT_SET_TYPE,new String());
		table.put(VOCLEditorConstants.PREDICAT_TYPE,new Integer(0));
	}
	
	/**
	 * Retrieves the dimension of the constraint
	 * @return - the dimension
	 */
	public Dimension getDim() {
		return dim;
	}

	/**
	 * Sets the new dimension of the constraint
	 * @param dimension
	 */
	public void setDim(Dimension dimension) {
		dim = dimension;
	}

	/**
	 * @return
	 */
	public Hashtable getValues() {
		initValues(values);
		return values;
	}

	/**
	 * @param hashtable
	 */
	public void setValues(Hashtable hashtable) {
		values.clear();
		values.putAll(hashtable);
	}

	/**
	 * @param container
	 */
	public void setNavcontainer(NavExpContainer container) {
		navcontainer = container;
		constraint=null;
		VOCLExpression exp=(VOCLExpression)navcontainer;
		while(constraint==null){
			if(exp.eContainer() instanceof Constraint)
				constraint=(Constraint) exp.getConstraint();
			else exp=(VOCLExpression) exp.eContainer();	
		}		
	}

}
