/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.commands.delete;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.gef.commands.Command;

import vocleditor.model.VOCLModelManager;
import VOCLModel.BoolExp;
import VOCLModel.Constraint;
import VOCLModel.IfExp;
import VOCLModel.LetExp;
import VOCLModel.NavExpContainer;
import VOCLModel.PredicateContainer;
import VOCLModel.VOCLDiagramm;
import VOCLModel.VOCLExpression;
import VOCLModel.VOCLModel4Factory;


/**
 * @author eddi
 */
public class DeleteVOCLExpressionCommand extends Command {

	private VOCLExpression expression;
	//private VOCLExpression container;
	private Object container;
	private List vardec=new LinkedList();
	private Constraint constraint;
	private NavExpContainer navexp;
	private DeleteSetClassifierRole deleteSetClassifier;
	DeleteVOCLExpression deleteExp;

	/**
	 * Ths constructor of this command
	 */
	public DeleteVOCLExpressionCommand() {
		super();
	}

	/**
	 * The constructor of this command with a specified label
	 * @param label - the label indicating this command
	 */
	public DeleteVOCLExpressionCommand(String label) {
		super(label);
	}

	/**
	 * The execution of this command deletes a complete 
	 * VOCLExpression
	 */
	public void execute() {
		if(expression instanceof Constraint){
			((VOCLDiagramm)container).getConstraint().remove(expression);
			return;
		}
		vardec.clear();
		deleteExp=new DeleteVOCLExpression(vardec);
		deleteExp.deleteExpression(expression);
		constraint=null;
		VOCLExpression exp=expression;
		while(constraint==null){
			if(exp.eContainer() instanceof Constraint)
				constraint=(Constraint) exp.eContainer();
			else 
				exp=(VOCLExpression) exp.eContainer();
		}
		constraint.getVariableDeclaration().removeAll(vardec);
		
		if(expression instanceof PredicateContainer){
			PredicateContainer predicate=(PredicateContainer) expression;
			deleteSetClassifier=new DeleteSetClassifierRole(constraint,
															predicate.getPredicat().getCallVar(),
															(NavExpContainer) predicate.eContainer(),
															predicate.getPredicat());
			deleteSetClassifier.execute();
			return;
		}
		VOCLModel4Factory mfc = VOCLModelManager.getFactory();
		navexp=mfc.createNavExpContainer();
		navexp.setExptype(expression.getExptype());
		setInContainer(navexp);
		
		
	}
	
	
	//delete or put expression in the container
	private void setInContainer(VOCLExpression exp){
		switch(exp.getExptype().getValue()){
		case 0:
			((Constraint)container).setVOCLExpression(null);
			((Constraint)container).setVOCLExpression(exp);
			break;
		case 1:
			((BoolExp)container).setOp1(null);
			((BoolExp)container).setOp1(exp);
			break;
		case 2:
			((BoolExp)container).setOp2(null);
			((BoolExp)container).setOp2(exp);
			break;
		case 3:
			((IfExp)container).setIfExpression(null);
			((IfExp)container).setIfExpression(exp);
			break;
		case 4:
			((IfExp)container).setThenExpression(null);
			((IfExp)container).setThenExpression(exp);
			break;
		case 5:
			((IfExp)container).setElseExpression(null);
			((IfExp)container).setElseExpression(exp);
			break;
		case 6:
			((LetExp)container).setLetExpression(null);
			((LetExp)container).setLetExpression(exp);
			break;
		case 7:
			((LetExp)container).setInExpression(null);
			((LetExp)container).setInExpression(exp);		
		}
	}
	
	/**
	 * Redos a deletion of a VOCLExpression
	 */
	public void redo() {
		if(expression instanceof Constraint){
			((VOCLDiagramm)container).getConstraint().remove(expression);
			return;
		}
		constraint.getVariableDeclaration().removeAll(vardec);
		if(expression instanceof PredicateContainer){
			deleteSetClassifier.execute();
			return;
		}
		setInContainer(navexp);
		deleteExp.getListIsIn().deleteAllIsIn();
	}

	/**
	 * Undos the deletion of a VOCLExpression
	 */
	public void undo() {
		if(expression instanceof Constraint){
			((VOCLDiagramm)container).getConstraint().add(expression);
			return;
		}
		constraint.getVariableDeclaration().addAll(vardec);
		if(expression instanceof PredicateContainer){
			deleteSetClassifier.undo();
			return;
		}
		setInContainer(expression);
		deleteExp.getListIsIn().createAllIsIn();
	}


	/**
	 * Expression to delete
	 * @param expression
	 */
	public void setExpression(VOCLExpression expression) {
		this.expression = expression;
	}

	

	/**
	 * Container,where the expression will be delete 
	 * @param container
	 */
	public void setContainer(Object container) {
		this.container = container;
	}


}
