/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.edit;


import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.ui.views.properties.IPropertySource;

import vocleditor.edit.policy.ClassifierRoleComponentEditPolicy;
import vocleditor.edit.policy.ClassifierRoleEditPolicy;
import vocleditor.figure.AbstractNodeFigure;
import vocleditor.figure.SetClassifierRoleFigure;
import vocleditor.figure.ClassifierRoleFigure;
import vocleditor.figure.VoclAnchor;
import vocleditor.model.EObjectPropertySource;
import VOCLModel.AssociationEndRoleSource;
import VOCLModel.AssociationEndRoleTarget;
import VOCLModel.AssociationRole;
import VOCLModel.ClassifierNode;
import VOCLModel.VOCLModel4Package;

/**
 * Class       :AbstractClassifierNodeEditPart<br/>
 * Package     : vocleditor.edit<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Class to provide application for ClassifierNode
 * <br/>
 * @author vilapower
 */
public abstract class AbstractClassifierNodeEditPart
	extends AbstractGraphicalEditPart implements NodeEditPart, Adapter{

	private IPropertySource propertySource = null;
	private Notifier target;


	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {return null;}
	
	/** 
	 * Return the Model of this EditPart as ClassifierNode.
	 * @see VOCLModel3.ClassifierNode
	 *@return  ClassifierNode .
	 */
	public ClassifierNode getClassifierNode() {
		return (ClassifierNode)getModel();
	}
	
	/** 
	 * Returns the Figure of this, as a node type figure.
	 * @return  Figure as a AbstractNodeFigure.
	 */
	protected AbstractNodeFigure getNodeFigure(){
		return (AbstractNodeFigure) getFigure();
	}
	
	/**
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 * Gets the source connection anchor of a Association Role. Returns the nearest anchor 
	 * if there is no connection or 
	 * 
	 */
	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) { 	
		VoclAnchor sourceAnchor;	
		// Connection has no AssociationEndRole at source 
		if (((AssociationRole)connection.getModel()).getSource() == null){			
			// Fallunterscheidung
			if(getFigure() instanceof ClassifierRoleFigure)
		    	sourceAnchor = ((ClassifierRoleFigure) getFigure()).getSourceConnectionAnchorAt();
		    else sourceAnchor = ((SetClassifierRoleFigure) getFigure()).getSourceConnectionAnchorAt();	
			return sourceAnchor;
		}else {
			AssociationEndRoleSource  eRS = ((AssociationRole)connection.getModel()).getSource();
			int nr = eRS.getNumberOfAnchor();
			if(getFigure() instanceof ClassifierRoleFigure)
				sourceAnchor = (VoclAnchor) ((ClassifierRoleFigure) getFigure()).getSourceConnectionAnchors().get(nr);
			else sourceAnchor = (VoclAnchor) ((SetClassifierRoleFigure) getFigure()).getSourceConnectionAnchors().get(nr);
			return sourceAnchor;			
		}
	}
	
	
	/**
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.Request)
	 */
	public ConnectionAnchor getSourceConnectionAnchor(Request request) {		
		Figure f = (Figure) getFigure();
		ConnectionAnchor sC;
		if (f instanceof ClassifierRoleFigure ){
			((ClassifierRoleFigure) f).setMousePoint(request);
			sC = ((ClassifierRoleFigure) f).getSourceConnectionAnchorAt();			
		}else{
			((SetClassifierRoleFigure)f).setMousePoint(request);
			sC = ((SetClassifierRoleFigure)f).getSourceConnectionAnchorAt();
		}
		return sC;
	}

	/**
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 */
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {
		VoclAnchor targetAnchor;	
			// Connection has no AssociationEndRole at source  
			if (((AssociationRole)connection.getModel()).getTarget() == null){	
		
				// Fallunterscheidung
				if(getFigure() instanceof ClassifierRoleFigure)
						targetAnchor = ((ClassifierRoleFigure) getFigure()).getTargetConnectionAnchorAt();
				else targetAnchor = ((SetClassifierRoleFigure) getFigure()).getTargetConnectionAnchorAt();
		
				return targetAnchor;
			}else {
				AssociationEndRoleTarget  eRT = ((AssociationRole)connection.getModel()).getTarget();
				int nr = eRT.getNumberOfAnchor();
				if(getFigure() instanceof ClassifierRoleFigure)
					targetAnchor = (VoclAnchor) ((ClassifierRoleFigure) getFigure()).getTargetConnectionAnchors().get(nr);
				else targetAnchor = (VoclAnchor) ((SetClassifierRoleFigure) getFigure()).getTargetConnectionAnchors().get(nr);
				
				return targetAnchor;			
			}
	}
	

	/**
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
	 */
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
	
		Figure f = (Figure) getFigure();
		ConnectionAnchor tC;
		if (f instanceof ClassifierRoleFigure ){
			((ClassifierRoleFigure) f).setMousePoint(request);
			tC = ((ClassifierRoleFigure) f).getTargetConnectionAnchorAt();			
		}else{
			((SetClassifierRoleFigure)f).setMousePoint(request);
			tC = ((SetClassifierRoleFigure)f).getTargetConnectionAnchorAt();
		}
		return tC;
	}


	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelSourceConnections()
	 */
	protected List getModelSourceConnections() {
		List list=new LinkedList();
		Iterator it;
		it = getClassifierNode().getSource().iterator();
		while( it.hasNext() )
		list.add( ((AssociationEndRoleSource)it.next()).getAssociationRole());
		return list;
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
	 */
	protected List getModelTargetConnections() {
		List list=new LinkedList();
		Iterator it;
		it = getClassifierNode().getTarget().iterator();
		while( it.hasNext() )
		list.add( ((AssociationEndRoleTarget)it.next()).getAssociationRole() );
		return list;
	}
	

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		// install the edit policy to handle connection creation
		installEditPolicy( EditPolicy.GRAPHICAL_NODE_ROLE, new ClassifierRoleEditPolicy() );
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new ClassifierRoleComponentEditPolicy());
	}
	
	/**
	* when AssociationsEndRoles are added to a ClassifierNode, add this EditPart as a listener on the port so
	* that it gets notified of AssociationRole being added or removed. When Association are added
	*  or removed from a EndRole owned by the SimpleClassifierRole of this EditPart, refresh the connections.
	* @param notification org.eclipse.emf.common.notify.Notification
	* 
	*/

	public void notifyChanged(Notification notification) {
		int type = notification.getEventType();
		int featureId = notification.getFeatureID(VOCLModel4Package.class);
		switch( featureId ) {
			case VOCLModel4Package.CLASSIFIER_NODE__SOURCE:		
				refreshSourceConnections();break;
			case VOCLModel4Package.CLASSIFIER_NODE__TARGET:
				refreshTargetConnections();break;
			case VOCLModel4Package.CLASSIFIER_NODE__Y:
				refreshVisuals();break;		
			case VOCLModel4Package.CLASSIFIER_NODE__X:
				//	if(notification.getNewValue() instanceof Integer)
						refreshVisuals();
					break;
			default:
				refreshVisuals();
		}		
	}


	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
		Point loc = new Point( getClassifierNode().getX(), getClassifierNode().getY() );
		Dimension size = new Dimension(-1,-1);
		Rectangle r = new Rectangle(loc ,size);
		((GraphicalEditPart) getParent()).setLayoutConstraint(
			this,
			getFigure(),
			r);
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	public void activate(){
		if (isActive()) return;
		((Notifier)getClassifierNode()).eAdapters().add(this);	
		super.activate();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate(){
		if (!isActive()) return;
		((Notifier)getClassifierNode()).eAdapters().remove(this);	
		super.deactivate();
	}


	/**
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/**
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		return type.equals( getModel().getClass() );
	}


	/**
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		target = newTarget;
	}

	/**
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter( key );
	}
	
	/**
	 * returns IPropertySource for the EditPart
	 * @return IPropertySource 
	 */	
	protected IPropertySource getPropertySource() {
		if( propertySource == null ) {
			propertySource = new EObjectPropertySource( getClassifierNode() );
		}
		return propertySource;
	}

	

}
