/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.edit;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.views.properties.IPropertySource;

import vocleditor.edit.policy.ConstraintComponentEditPolicy;
import vocleditor.edit.policy.VOCLConstraintXYLayoutEditPolicy;
import vocleditor.edit.policy.VOCLEditPolicy;
import vocleditor.figure.ConstraintFigure;
import vocleditor.model.EObjectPropertySource;
import VOCLModel.Constraint;
import VOCLModel.Context;
import VOCLModel.VOCLModel4Package;
import VOCLModel.VariableDeclaration;



/**
 * Class       :ConstraintEditPart<br/>
 * Package     : vocleditor.edit<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Class to provide application for Constraint
 * <br/>
 * @author vilapower
 */
public class ConstraintEditPart extends AbstractGraphicalEditPart implements Adapter  {
	private IPropertySource propertySource = null;
	private Notifier target;
	private Shell ssssshell;
	private ConstraintFigure conFig;
	/**
	 * @param element
	 */
	public ConstraintEditPart(Constraint container) {
		setModel(container);
	}


	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure()
	{
		Constraint constraint= getConstraint();
		Context context=constraint.getContext();
		String strContext="Context ";
		if(context.isIsClassifier()){
			VariableDeclaration varDec= (VariableDeclaration)context.getVariableDeclaration().get(0);
			if (varDec.getVarName().equals("")){
				strContext=strContext + varDec.getType();	
			}
			else {
				strContext=strContext + varDec.getVarName() + " :" + varDec.getType();
			}
		}else{
			strContext=strContext + " :" + context.getName()+ "(";
			List parameter=context.getVariableDeclaration();
			for(int i=0 ; i<parameter.size();i++){
				VariableDeclaration varDec=(VariableDeclaration)parameter.get(i);
				strContext=strContext + varDec.getVarName() + " :" + varDec.getType()+ " ";
			}				
			strContext=strContext + ") :" + context.getType();
		}
		strContext=strContext + " " + constraint.getKind()+ ": " + constraint.getName();
		conFig = new ConstraintFigure(strContext);
		return conFig;
	}
	
	public ConstraintFigure getConstraintFigure(){
		return (ConstraintFigure)getFigure();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies(){
		installEditPolicy(EditPolicy.CONTAINER_ROLE,  new VOCLEditPolicy());	
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new VOCLConstraintXYLayoutEditPolicy());
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new ConstraintComponentEditPolicy());
	}

	/**
	 * Returns the element.
	 * @return the element
	 */
	public Constraint getConstraint()
	{
		return (Constraint) getModel();
	}
    
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	protected List getModelChildren()
	{
		List	allChildren = new LinkedList();	
		if(getConstraint().getVOCLExpression()!=null)
			allChildren.add(getConstraint().getVOCLExpression());
		return allChildren;
	}

	
	

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	public void activate()
	{
		if (isActive())
			return;

		// start listening for changes in the model
		getConstraint().eAdapters().add(this);

		super.activate();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate()
	{
		if (!isActive())
			return;

		// stop listening for changes in the model
		getConstraint().eAdapters().remove(this);

		super.deactivate();
	}



	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		return type.equals( getModel().getClass() );
	}


	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		target = newTarget;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
		/* override the default behavior defined in AbstractEditPart
		*  which would expect the model to be a property sourced. 
		*  instead the editpart can provide a property source
		*/
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter( key );
	}
	

	
	
	protected IPropertySource getPropertySource() {
		if( propertySource == null ) {
			propertySource = new EObjectPropertySource( getConstraint() );
		}
		return propertySource;
		
	}
	
	public void notifyChanged(Notification notification) {
		int type = notification.getEventType();
		int featureId = notification.getFeatureID(VOCLModel4Package.class);
		switch(featureId){

		case VOCLModel4Package.CONSTRAINT__NAME:
			refreshChildren();
			refreshVisuals();
			break;
		case VOCLModel4Package.CONSTRAINT__HEIGHT:
		case VOCLModel4Package.CONSTRAINT__WIDTH:	
			refreshChildren();	
			refreshVisuals();
			break;
		case VOCLModel4Package.CONSTRAINT__VOCL_EXPRESSION:		
			refreshChildren();
			break;
		default:
			refreshVisuals();
		}
		
	}
	
	
	protected void refreshVisuals() {
		Point loc = new Point( getConstraint().getX(), getConstraint().getY() );
		Dimension size=new Dimension(getConstraint().getWidth(),
							  		 getConstraint().getHeight());
		Rectangle r = new Rectangle(loc ,size);
		((GraphicalEditPart) getParent()).setLayoutConstraint(
			this,
			getFigure(),
			r);

	}


	/**
	 * @see org.eclipse.gef.GraphicalEditPart#getContentPane()
	 */
	public IFigure getContentPane() {
		return getConstraintFigure().getBody();
	}

}
