/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.edit;


import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;

import VOCLModel.AssociationEndRoleSource;
import VOCLModel.AssociationEndRoleTarget;
import VOCLModel.AssociationIsIn;
import VOCLModel.AssociationNav;
import VOCLModel.BoolExp;
import VOCLModel.ClassifierRole;
import VOCLModel.Condition;
import VOCLModel.Constraint;
import VOCLModel.IfExp;
import VOCLModel.LetExp;
import VOCLModel.NavExpContainer;
import VOCLModel.PredicateContainer;
import VOCLModel.SetClassifierRole;
import VOCLModel.VOCLDiagramm;
import VOCLModel.VOCLSash;




public class GraphicalEditPartsFactory implements EditPartFactory{

	
    /**
     * @see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart, java.lang.Object)
     */
    public GraphicalEditPartsFactory(){
    	super();
    }
    
    public EditPart createEditPart(EditPart context, Object obj)
    {
		if(obj instanceof VOCLDiagramm)
			return new VOCLDiagrammEditPart((VOCLDiagramm)obj);
		else if(obj instanceof NavExpContainer)
			return new NavExpContainerEditPart((NavExpContainer)obj);
        else if(obj instanceof Condition)
			return  new ConditionEditPart((Condition)obj);
		else if(obj instanceof ClassifierRole)
			return new ClassifierRoleEditPart((ClassifierRole)obj);
		else if(obj instanceof SetClassifierRole)
			return new SetClassifierRoleEditPart((SetClassifierRole)obj);
		else if(obj instanceof AssociationNav)
			return new AssociationRoleEditPart((AssociationNav)obj);
		else if(obj instanceof AssociationIsIn)
			return new AssociationRoleEditPart((AssociationIsIn)obj);
		else if(obj instanceof Constraint)
			return  new ConstraintEditPart((Constraint)obj);
	  	else if(obj instanceof AssociationEndRoleSource)
			return new AssociationEndRoleEditPart((AssociationEndRoleSource)obj);
		else if(obj instanceof AssociationEndRoleTarget)
			return new AssociationEndRoleEditPart((AssociationEndRoleTarget)obj);				
		else if(obj instanceof BoolExp)
			return new BoolExpEditPart((BoolExp)obj);
		else if(obj instanceof LetExp)
			return new LetExpEditPart((LetExp)obj);				
		else if(obj instanceof VOCLSash)
			return new VOCLSashEditPart((VOCLSash)obj);				
		else if(obj instanceof IfExp)
			return new IfExpEditPart((IfExp)obj);	
		else if(obj instanceof PredicateContainer)
			return new PredicatContainerEditPart((PredicateContainer)obj);	
        return null;
    }

}
