/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.edit;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.views.properties.IPropertySource;

import vocleditor.edit.policy.ExpressionComponentEditPolicy;
import vocleditor.edit.policy.PredicatContainerEditPolicy;
import vocleditor.edit.policy.PredicatContainerXYLayoutEditPolicy;
import vocleditor.figure.ConstraintFigure;
import vocleditor.figure.PredicatContainerFigure;
import vocleditor.model.EObjectPropertySource;
import VOCLModel.PredicateContainer;
import VOCLModel.VOCLModel4Package;
import VOCLModel.VariableDeclaration;



/**
 * Class       :SelectionContainerEditPart<br/>
 * Package     : vocleditor.edit<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Class to provide application for Constraint
 * <br/>
 * @author vilapower
 */
public class PredicatContainerEditPart extends AbstractGraphicalEditPart implements Adapter  {
	private IPropertySource propertySource = null;
	private Notifier target;
	private Shell ssssshell;
	private ConstraintFigure conFig;
	/**
	 * @param element
	 */
	public PredicatContainerEditPart(PredicateContainer container) {
		setModel(container);
	}


	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure(){
		PredicatContainerFigure fig= new PredicatContainerFigure();
		fig.setPredicatType(getSelectionContainer().getPredType().getValue());
		VariableDeclaration var=getSelectionContainer().getIterator();
		String name=new String("");
		if(var!=null)name =var.getVarName();
		fig.setIterator(name);
		
		return fig;
	}
	
	public PredicatContainerFigure getSelectionContainerFigure(){
		return (PredicatContainerFigure)getFigure();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies(){
		installEditPolicy(EditPolicy.CONTAINER_ROLE,  new PredicatContainerEditPolicy());	
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new PredicatContainerXYLayoutEditPolicy());
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new ExpressionComponentEditPolicy());
	}

	/**
	 * Returns the element.
	 * @return the element
	 */
	public PredicateContainer getSelectionContainer(){
		return (PredicateContainer) getModel();
	}
    
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	protected List getModelChildren(){
		List	allChildren = new LinkedList();	
		if(getSelectionContainer().getVOCLExpression()!=null)
			allChildren.add(getSelectionContainer().getVOCLExpression());
		return allChildren;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	public void activate(){
		if (isActive())return;

		// start listening for changes in the model
		getSelectionContainer().eAdapters().add(this);
		super.activate();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate(){
		if (!isActive())return;

		// stop listening for changes in the model
		getSelectionContainer().eAdapters().remove(this);
		super.deactivate();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		return type.equals( getModel().getClass() );
	}

	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		target = newTarget;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
		/* override the default behavior defined in AbstractEditPart
		*  which would expect the model to be a property sourced. 
		*  instead the editpart can provide a property source
		*/
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter( key );
	}
	
	protected IPropertySource getPropertySource() {
		if( propertySource == null ) {
			propertySource = new EObjectPropertySource( getSelectionContainer() );
		}
		return propertySource;
		
	}
	
	public void notifyChanged(Notification notification) {
		int type = notification.getEventType();
		int featureId = notification.getFeatureID(VOCLModel4Package.class);
		switch(featureId){

		case VOCLModel4Package.PREDICATE_CONTAINER__HEIGHT:
		case VOCLModel4Package.PREDICATE_CONTAINER__WIDTH:	
			refreshChildren();	
			refreshVisuals();
			break;
		case VOCLModel4Package.PREDICATE_CONTAINER__VOCL_EXPRESSION:		
			refreshChildren();
			break;
		default:
			refreshVisuals();
		}
		
	}
	
	protected void refreshVisuals() {
		Point loc = new Point( getSelectionContainer().getX(), getSelectionContainer().getY() );
		Dimension size=new Dimension(getSelectionContainer().getWidth(),
							  		 getSelectionContainer().getHeight());
		Rectangle r = new Rectangle(loc ,size);
		PredicatContainerFigure fig=(PredicatContainerFigure)getFigure();
		fig.setPredicatType(getSelectionContainer().getPredType().getValue());
		
		VariableDeclaration var=getSelectionContainer().getIterator();
		String name=new String("");
		if(var!=null)name =var.getVarName();
		fig.setIterator(name);	
		
		((GraphicalEditPart) getParent()).setLayoutConstraint(
			this,
			getFigure(),
			r);

	}


	/**
	 * @see org.eclipse.gef.GraphicalEditPart#getContentPane()
	 */
	public IFigure getContentPane() {
		return getSelectionContainerFigure().getBody();
	}

}
