/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.edit;



import org.eclipse.draw2d.BorderLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.DragTracker;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.ui.views.properties.IPropertySource;

import vocleditor.commands.constraints.SashTracker;
import vocleditor.figure.SashFigure;
import vocleditor.model.EObjectPropertySource;
import VOCLModel.VOCLSash;

/**
 * Class       :VOCLSashEditPart<br/>
 * Package     : vocleditor.edit<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Class to provide application for VOCLSash
 * <br/>
 * @author vilapower
 */

public class VOCLSashEditPart extends AbstractGraphicalEditPart implements Adapter{
	
	private IPropertySource propertySource = null;
	private Notifier target;


	/**
	 * Creates a new VOCLSashEditPart instance.<br/>
	 * @param element VOCLSash
	 */
	public VOCLSashEditPart(VOCLSash sash){
		setModel(sash);
	}
	
	/** 
	 * This method instanciates and returns a SashFigure <br/>
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()<br/>
	 * @see SashFigure;
	 */
	protected IFigure createFigure(){
		VOCLSash sash = getSash();	
		return new SashFigure();
	}

	/** 
	 * Returns the SashFigure 
	 * @return  SashFigure 
	 */
	public SashFigure getSashFigure(){
		return (SashFigure) getFigure();
	}
	
	/**
	*checks ,if the parameters of VOCLSash were changed
	* @param notification org.eclipse.emf.common.notify.Notification
	*/
	public void notifyChanged(Notification notification){
		refreshVisuals();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies(){}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals(){
		((GraphicalEditPart) getParent()).setLayoutConstraint(this,
															  getFigure(),
															  BorderLayout.CENTER);
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	public void activate(){
		if (isActive())return;
		// start listening for changes in the model
		 ((Notifier)getSash()).eAdapters().add(this);
		super.activate();
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate(){
		if (!isActive()) return;
		// stop listening for changes in the model
		 ((Notifier)getSash()).eAdapters().remove(this);
		super.deactivate();
	}


	private VOCLSash getSash(){
		return (VOCLSash)getModel();
	}
	
	/**
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget(){
		return target;
	}

	/**
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type){
		return type.equals(getModel().getClass());
	}
	
	/**
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget){
		target = newTarget;
	}
	
	/**
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key){
		if (IPropertySource.class == key){
			return getPropertySource();
		}
		return super.getAdapter(key);
	}

	/**
	 * returns IPropertySource for the EditPart
	 * @return IPropertySource 
	 */	
	protected IPropertySource getPropertySource(){
		if (propertySource == null){
			propertySource = new EObjectPropertySource(getSash());
		}
		return propertySource;
	}


	/**
	 * @see org.eclipse.gef.EditPart#getDragTracker(org.eclipse.gef.Request)
	 */
	public DragTracker getDragTracker(Request request) {
		return new SashTracker(this,getParent());
	}



}
