/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.edit.policy;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;

import vocleditor.commands.connect.IsInConnectCommand;
import vocleditor.commands.connect.NavConnectCommand;
import vocleditor.commands.connect.ReConnectCommand;
import vocleditor.edit.AbstractClassifierNodeEditPart;
import vocleditor.edit.AssociationRoleEditPart;
import vocleditor.figure.VoclAnchor;
import VOCLModel.AssociationEndRole;
import VOCLModel.AssociationIsIn;
import VOCLModel.AssociationNav;
import VOCLModel.AssociationRole;
import VOCLModel.ClassifierNode;
import VOCLModel.NavExpContainer;




public class ClassifierRoleEditPolicy extends GraphicalNodeEditPolicy {


	public ClassifierRoleEditPolicy() {
		super();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCompleteCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getConnectionCompleteCommand(CreateConnectionRequest request) {
		if (request.getStartCommand() instanceof NavConnectCommand){	
			NavConnectCommand command = (NavConnectCommand)request.getStartCommand();
			VoclAnchor editorAnchor = (VoclAnchor) getClassifierNodeEditPart().getTargetConnectionAnchor(request); 
			command.setNumberOfTargetAnchor(editorAnchor.getId());
		    command.setTarget(getClassifierNode());
		    return command;
		}else if (request.getStartCommand() instanceof IsInConnectCommand){	
			IsInConnectCommand command = (IsInConnectCommand)request.getStartCommand();
			VoclAnchor editorAnchor = (VoclAnchor) getClassifierNodeEditPart().getTargetConnectionAnchor(request); 
			command.setNumberOfTargetAnchor(editorAnchor.getId());
		    command.setTarget(getClassifierNode());
		    return command;
		}
		return null;
	
	}
	
	protected ClassifierNode getClassifierNode() {
		return (ClassifierNode)getHost().getModel();
	}

	protected AbstractClassifierNodeEditPart getClassifierNodeEditPart() {
		return (AbstractClassifierNodeEditPart)getHost();
	}
	
	/**
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getConnectionCreateCommand(CreateConnectionRequest request) {
		if(request.getNewObject() instanceof AssociationNav){
			NavConnectCommand command = new NavConnectCommand();
			command.setContainer((NavExpContainer)getClassifierNodeEditPart().getParent().getModel());
			command.setAssociation((AssociationNav)request.getNewObject());
			command.setSource(getClassifierNode());
			VoclAnchor editorAnchor = (VoclAnchor) getClassifierNodeEditPart().getSourceConnectionAnchor(request); 
			command.setNumberOfSourceAnchor(editorAnchor.getId());
		    request.setStartCommand(command);
		    return command;
		}else if(request.getNewObject() instanceof AssociationIsIn){
			IsInConnectCommand command = new IsInConnectCommand();
			command.setContainer((NavExpContainer)getClassifierNodeEditPart().getParent().getModel());
			command.setAssociation((AssociationIsIn)request.getNewObject());
			command.setSource(getClassifierNode());
			VoclAnchor editorAnchor = (VoclAnchor) getClassifierNodeEditPart().getSourceConnectionAnchor(request); 
			command.setNumberOfSourceAnchor(editorAnchor.getId());
		    request.setStartCommand(command);
		    return command;
		}
		return null;
	
	}

	/**
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectTargetCommand(org.eclipse.gef.requests.ReconnectRequest)
	 */
	protected Command getReconnectTargetCommand(ReconnectRequest request) {
		if(request.getConnectionEditPart() instanceof AssociationRoleEditPart){
					ClassifierNode cN = (ClassifierNode) ((AbstractClassifierNodeEditPart)request.getTarget()).getModel();
					ReConnectCommand command = new ReConnectCommand();
					VoclAnchor editorAnchor = (VoclAnchor) ((AbstractClassifierNodeEditPart)request.getTarget()).getTargetConnectionAnchor(request);
					command.setNumberOfAnchor(editorAnchor.getId());
					AssociationEndRole aR =((AssociationRole)((AssociationRoleEditPart)request.getConnectionEditPart()).getModel()).getTarget();
					command.setAssERS(aR);			
					command.setNode(cN);					
					return command;
				}
		return null;		
	}

	/**
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectSourceCommand(org.eclipse.gef.requests.ReconnectRequest)
	 */
	protected Command getReconnectSourceCommand(ReconnectRequest request) {
		if(request.getConnectionEditPart() instanceof AssociationRoleEditPart){
					ClassifierNode cN = (ClassifierNode) ((AbstractClassifierNodeEditPart)request.getTarget()).getModel();
					ReConnectCommand command = new ReConnectCommand();
					VoclAnchor editorAnchor = (VoclAnchor) ((AbstractClassifierNodeEditPart)request.getTarget()).getSourceConnectionAnchor(request);
					command.setNumberOfAnchor(editorAnchor.getId());
					AssociationEndRole aR = ((AssociationRole)((AssociationRoleEditPart)request.getConnectionEditPart()).getModel()).getSource();
					command.setAssERS(aR);			
					command.setNode(cN);					
					return command;
				}
		
		return null;
	}
}
