/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.editors;

//import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.EditDomain;
//import org.eclipse.gef.EditDomain;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.ui.parts.GraphicalViewerKeyHandler;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import vocleditor.edit.GraphicalEditPartsFactory;


public class VOCLPage extends AbstractEditorPage
{
    /**
     * Creates a new VOCLPage instance.
     * 
     * <p>By design this page uses its own <code>EditDomain</code>.
     * The main goal of this approach is that this page has its own
     * undo/redo command stack.
     * 
     * @param parent the parent multi page editor 
     */
    public VOCLPage(VOCLEditor parent)
    {
        super(parent, new EditDomain());
    }

    /* (non-Javadoc)
     * @see com.ibm.itso.sal330r.gefdemo.editor.AbstractEditorPage#getPageName()
     */
    protected String getPageName()
    {

        return "Visual-OCL";

    }

    /* (non-Javadoc)
     * @see com.ibm.itso.sal330r.gefdemo.editor.AbstractEditorPage#createPageControl(org.eclipse.swt.widgets.Composite)
     */
    protected void createPageControl(Composite parent)
    {
        Composite composite = new Composite(parent, SWT.NONE);
        composite.setBackground(parent.getBackground());
        composite.setLayout(new GridLayout(2, false));
        
        createPaletteViewer(composite);
        GridData gd = new GridData(GridData.FILL_VERTICAL);
        gd.widthHint = 125;
        getPaletteViewer().getControl().setLayoutData(gd);
        
        createGraphicalViewer(composite);
        gd = new GridData(GridData.FILL_BOTH);
        gd.widthHint = 275;
        getGraphicalViewer().getControl().setLayoutData(gd);
    }

    /** the graphical viewer */
    private GraphicalViewer viewer;

    /**
     * Creates the GraphicalViewer on the specified <code>Composite</code>.
     * @param parent the parent composite
     */
    private void createGraphicalViewer(Composite parent)
    {
        viewer = new ScrollingGraphicalViewer();
        viewer.createControl(parent);
        // configure the viewer
        viewer.getControl().setBackground(parent.getBackground());
        viewer.setRootEditPart(new ScalableFreeformRootEditPart());
        viewer.setKeyHandler(new GraphicalViewerKeyHandler(viewer));

        // hook the viewer into the editor
        registerEditPartViewer(viewer);

        // configure the viewer with drag and drop
        configureEditPartViewer(viewer);

        // initialize the viewer with input
        viewer.setEditPartFactory(new GraphicalEditPartsFactory());
        viewer.setContents(getVOCLEditor().getVOCLDiagramm());
        
		ContextMenuProvider provider = new VOCLEditorMenuProvider(getGraphicalViewer(),
				this.getVOCLEditor().getActionRegistry());
			 getGraphicalViewer().setContextMenu(provider);
			 getSite().registerContextMenu(provider,getGraphicalViewer());
//		END insertion
    }

    /* (non-Javadoc)
     * @see com.ibm.itso.sal330r.gefdemo.editor.AbstractEditorPage#getGraphicalViewerForZoomSupport()
     */
    protected GraphicalViewer getGraphicalViewer()
    {
        return viewer;
    }
    
    
	
    
    
	/* (non-Javadoc)
	 * @see vocleditor.editors.AbstractEditorPage#getPaletteRoot()
	 */
	protected PaletteRoot getPaletteRoot() {
		return super.getPaletteRoot();
	}

}
