/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */

package vocleditor.editors;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.palette.MarqueeToolEntry;
import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.palette.PaletteEntry;
import org.eclipse.gef.palette.PaletteGroup;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.palette.PaletteSeparator;
import org.eclipse.gef.palette.SelectionToolEntry;
import org.eclipse.gef.palette.ToolEntry;

import vocleditor.editors.tools.VOCLConnectionCreationToolEntry;
import vocleditor.editors.tools.VOCLCreationToolEntry;
import vocleditor.model.ModelCreationFactory;
import VOCLModel.AssociationIsIn;
import VOCLModel.AssociationNav;
import VOCLModel.BoolExp;
import VOCLModel.ClassifierRole;
import VOCLModel.Constraint;
import VOCLModel.IfExp;
import VOCLModel.LetExp;
import VOCLModel.PredicateContainer;
import VOCLModel.SetClassifierRole;

public class VOCLPaletteRoot extends PaletteRoot 
{

    /**
     * Creates a new VOCLPaletteRoot instance.
     * 
     */
    public VOCLPaletteRoot()
    {
        // create root
        super();
        
        // a group of default control tools
        PaletteGroup controls = new PaletteGroup("Controls");

        // the selection tool
        ToolEntry tools = new SelectionToolEntry();
        controls.add(tools);

        // use selection tool as default entry
        setDefaultEntry(tools);
		// the marquee selection tool
		controls.add(new MarqueeToolEntry());
        
		ToolEntry tool =new VOCLCreationToolEntry(
						  "Constraint",
						  "Creates a constraint",
						  new ModelCreationFactory(Constraint.class),
						  null,
						  null);
		controls.add(tool);
		
		List entries = new ArrayList();
		entries.add(tools);
			
		PaletteDrawer drawer = new PaletteDrawer("Container");


		entries = new ArrayList();
	
		
		tool =new VOCLCreationToolEntry(
						  "Boolean Expression",
						  "Creates a Boolean-container",
						  new ModelCreationFactory(BoolExp.class),
						  null,
						  null);
						  
		entries.add(tool);
		
		tool =new VOCLCreationToolEntry(
						  "Let Expression",
						  "Creates a Let-container",
						  new ModelCreationFactory(LetExp.class),
						  null,
						  null);
						  
		entries.add(tool);
		
		tool =new VOCLCreationToolEntry(
						  "IfThenElse Expression",
						  "Creates a IfThenElse-container",
						  new ModelCreationFactory(IfExp.class),
						  null,
						  null);
						  
		entries.add(tool);				
		
		tool =new VOCLCreationToolEntry(
						  "Predicate Container",
						  "Creates a predicate container",
						  new ModelCreationFactory(PredicateContainer.class),
						  null,
						  null);
						  
		entries.add(tool);	
		drawer.addAll(entries);
		
		
		add(controls);
		add(drawer);
		
		drawer = new PaletteDrawer("Expression");


		entries = new ArrayList();
	
		tool =new VOCLConnectionCreationToolEntry(
					"AssociationRole",
					"Creates a associationrole",
					new ModelCreationFactory(AssociationNav.class),
					null,
					null);
											
		entries.add(tool);
		
		tool =new VOCLConnectionCreationToolEntry(
				"IsIn Association",
				"Creates a IsIn associationrole",
				new ModelCreationFactory(AssociationIsIn.class),
				null,
				null);
										
		entries.add(tool);
		
		PaletteSeparator sep = new PaletteSeparator("Separator1"); 
		sep.setUserModificationPermission(PaletteEntry.PERMISSION_NO_MODIFICATION);
		entries.add(sep);
		
		tool =new VOCLCreationToolEntry(
					"ClassifierRole",
				 	"Creates a classifierrole",
					new ModelCreationFactory(ClassifierRole.class),
					null,
					null);
		entries.add(tool);
        
		tool =new VOCLCreationToolEntry(
				   	"Collection ClassifierRole",
					"Creates a classifierrole",
				    new ModelCreationFactory(SetClassifierRole.class),
				    null,
				    null);
		entries.add(tool);	
		
		drawer.addAll(entries);
		add(drawer);
    }
}
