/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.figure;


import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;

import vocleditor.editors.VOCLEditorConstants;
import VOCLModel.Method;
import VOCLModel.VariableDeclaration;


public class ClassifierRoleFigure 
	extends AbstractNodeFigure
{
	
	protected ClassifierRoleLabel figure;

	static private int M_SIZE=100;
	//Label for the Method
	protected Label callMethod= new Label("");
	private String methodString=new String("");
	private boolean meth=false;
	

	//Constants for ARROW
	private static final PointList ARROW = new PointList();
	public static final int ARROW_LENGTH =50;	
	private static final Dimension ARROW_START = new Dimension(6,6);
	private static final Dimension ARROW_END = new Dimension(10,7);	
	static {
		//setup arrow
		ARROW.addPoint(0,0);
		ARROW.addPoint(10, 0);
		ARROW.addPoint(5, 7);

	}			 
	

	
	/**
	 * Constructor of the ClassifierRoleFigure:
	 * Creates a Figure for a class with a class-name,
	 * a variable-name (optional - may be empty, but not null),
	 * and attributes and place them inside a UML-like
	 * box for classes.
	 * @param shell     - the shell where the figure is drawn on
	 * @param varname   - the variable-name for this class-instance
	 * @param classname - the class name
	 */
	public ClassifierRoleFigure(String varname,String classname){	
		
		super(varname, classname);

		//initilize  and add SimpleClassifierRoleFigure
		figure=new ClassifierRoleLabel(varname,classname);	
		figure.setLocation(new Point(0,0));
		figure.setSize(figure.getPreferredSize());
		add(figure);	
		
		callMethod.setBackgroundColor(VOCLEditorConstants.classColor);
		callMethod.setBorder(new LineBorder(ColorConstants.black,1));
		callMethod.setVisible(false);
		add(callMethod);
		createVoclEditorAnchors();
						
	}

	/**
	 * Paint the figures Background Box 
	 */
	protected void paintFigure(Graphics graphics) {
		//if no Method, then no figure for method
		if(isMethod()){	
			Rectangle r = getBounds().getCopy();
			graphics.translate(r.getLocation());	
			graphics.setBackgroundColor(ColorConstants.black);
			Dimension dim=figure.getPreferredSize();
			
			//ARROW_START
			Rectangle point=new Rectangle(
					new Point((dim.width-ARROW_START.width)/2,0),ARROW_START);
			graphics.fillOval(point);
			
			//ARROW_LINE
			graphics.drawLine(new Point(dim.width/2,ARROW_START.width/2),
							  new Point(dim.width/2,ARROW_LENGTH));
							  
			//ARROW_END
			Rectangle ar =new Rectangle(new Point((dim.width-ARROW_END.width)/2,
												   ARROW_LENGTH-ARROW_END.height),
												   ARROW_END);
			graphics.translate(ar.getLocation());
			graphics.fillPolygon(ARROW);
			
			callMethod.setText(methodString);
			callMethod.setLocation(new Point(dim.width/2+2,ARROW_START.height+ARROW_LENGTH/5));
			figure.setLocation(new Point(0,ARROW_LENGTH));
			callMethod.setVisible(true);
		}else{
			figure.setLocation(new Point(0,0));
			callMethod.setVisible(false);
		}
		
	}
	/**
	 * Retrieving the wanted Dimension for this figure.
	 * It is calculated by the sum of the dimension of the
	 * classFigure (at the top) and the dimension for the
	 * attributeFigure (bottom) in y-axis, and the max of
	 * the values from both for x-axis.
	 */
	public Dimension getPreferredSize(int wHint, int hHint) {
		figure.setSize(figure.getPreferredSize());
		Dimension dim1 = figure.getPreferredSize(wHint,hHint).getCopy();
		if(!isMethod())return dim1;
		Dimension dim2 = callMethod.getPreferredSize().getCopy();		
		callMethod.setSize(dim2);
		if(dim1.width/2>dim2.width+2)
			return new Dimension(dim1.width+2,dim1.height+ARROW_LENGTH);
		else return new Dimension(dim1.width/2 + dim2.width+2,dim1.height+ARROW_LENGTH);
	}

	/**
	 * Getter method for the CompartmentFigure
	 * nested in this class for the attributes.
	 * @return - the CompartmentFigure for the attributes.
	 */
	public CompartmentFigure getAttributesCompartment() {
		return figure.getAttributesCompartment();
	}

	/**
	 * A method for specifying the variable- and classname
	 * of this ClassifierRoleFigure.
	 * Changes only the label, but not the values.
	 * @param varname 	- the name of the variable
	 * @param classname	- the name of the class
	 */
	public void setName(String varname,String classname){
		figure.setName(varname,classname);
	}
	
		
	/**
	 * Retrieving the minimum Dimension
	 * @see getPreferredSize()
	 */
	public Dimension getMinimumSize(int wHint, int hHint) {return getPreferredSize();}


	
	/**
	 * This method adds an Attribute to the figure, 
	 * specified by name, type and value.
	 * Inserted as Label in the following format:
	 * <name>: "<value>" [<type>]
	 * @param name	- the attributes name
	 * @param type	- the attributes type (e.g. String)
	 * @param value	- the attributes value
	 */
	public void addAttribute(String name, String type, String value, boolean isValue) {
		figure.addAttribute(name,type,value,isValue);
	}
	
	/**
	 * This method adds  Attributes to the figure, 
	 * specified by name, type and value.
	 * Inserted as Label in the following format:
	 * <name>: "<value>" [<type>]
	 * @param list	- list of attributes
	 */
	public void setAttributes(List list) {
		figure.setAttributes(list);
	}
	
	/**
	 * This method adds an Attribute to the figure, 
	 * specified by name, type and value.
	 * Inserted as Label in the following format:
	 * <name>: "<value>" [<type>]
	 * @param name	- the attributes name
	 * @param type	- the attributes type (e.g. String)
	 * @param value	- the attributes value
	 */
	public void changeAttribute(int index, String name, String type, String value) {
		figure.changeAttribute(index,name,type,value);
	}
	
	/**
	 * This method removes an element from the Attributelist
	 * specified by index.
	 */
	public void removeAttribute(int index) {
		figure.removeAttribute(index);
	}
	
	/**
	 * This method will set the ClassVarName of this 
	 * classifier in the graphical view.
	 */
	public void setClassVarName(String name) {
		figure.setClassVarName(name);
	}
	
	/**
	 * @return
	 */
	public boolean isMethod() {
		return meth;
	}

	/**
	 * @return
	 */
	public String getMethodStr() {
		return methodString;
	}

	/**
	 * @param b
	 */
	public void setMethod(boolean b) {
		meth = b;
	}

	/**
	 * @param method
	 */
	public void setMethodStr(Method met) {
		methodString="";
		if(met.getRReturn()!=null){
			if(met.getRReturn().getVarName().trim().length()!=0)
				methodString =methodString + met.getRReturn().getVarName()+"=";
		}	
		methodString =methodString + met.getName()+"( ";
		Iterator it=met.getCall().iterator();
		while(it.hasNext()){
			VariableDeclaration variable=(VariableDeclaration)it.next();
			methodString =methodString+variable.getVarName()+":"+variable.getType()+ " ";
		}
		methodString =methodString + ") :" +met.getType();
	}

}
