/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.wizards;


import org.eclipse.core.resources.IFile;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.dialogs.WizardNewFileCreationPage;
import org.eclipse.ui.ide.IDE;

/**
 * The "New" wizard page allows setting the container for
 * the new file as well as the file name. The page
 * will only accept file name without the extension OR
 * with the extension that matches the expected one (vocl).
 */

public class SampleNewVOCLExpressionPage extends WizardNewFileCreationPage {
	private Text containerText;
	private Text fileText;
	private ISelection selection;
	
	
	private IWorkbench	workbench;
	private static int eCount = 1;


	/**
	 * Constructor for VOCLNewWizardPage.
	 * @param pageName
	 */
	public SampleNewVOCLExpressionPage(IWorkbench aWorkbench, IStructuredSelection selection) {		
		super("VOCLPage", selection);
		setTitle("VOCL Editor File");
		setDescription("This wizard creates a new file with *.vocl extension that can be opened by a vocleditor.");
		//this.setImageDescriptor(ImageDescriptor.createFromFile(FlowPlugin.class,"images/flowbanner.gif"));
		this.workbench = aWorkbench;
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) {	
		super.createControl(parent);
		this.setFileName("VOCLExpression" + eCount + ".vocl");
		Composite composite = (Composite)getControl();
		setPageComplete(validatePage());
	}
	
	public boolean finish() {
		IFile newFile = createNewFile();
		if (newFile == null) 
			return false;  // ie.- creation was unsuccessful

		// Since the file resource was created fine, open it for editing
		// iff requested by the user
		try {
			IWorkbenchWindow dwindow = workbench.getActiveWorkbenchWindow();
			IWorkbenchPage page = dwindow.getActivePage();
			if (page != null)
				IDE.openEditor(page, newFile, true); 
		} 
		catch (org.eclipse.ui.PartInitException e) {
			e.printStackTrace();
			return false;
		}
		eCount++;
		return true;
	}
}